# -*- coding: utf-8 -*-

from plone import api
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plonemeeting.restapi.serializer.base import BaseDXSerializeFolderToJson
from plonemeeting.restapi.serializer.base import serialize_attendees
from plonemeeting.restapi.serializer.base import serialize_extra_include_annexes
from plonemeeting.restapi.serializer.base import serialize_pod_templates
from plonemeeting.restapi.serializer.summary import PMBrainJSONSummarySerializer
from Products.PloneMeeting.content.meeting import IMeeting
from Products.PloneMeeting.utils import get_dx_field
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface


class BaseSerializeMeetingToJson(object):
    """ """

    def _available_extra_includes(self, result):
        """ """
        result["@extra_includes"] = ["annexes", "attendees", "pod_templates"]
        return result

    def _extra_include(self, result):
        """ """
        extra_include = self._get_asked_extra_include()
        if "pod_templates" in extra_include:
            result["extra_include_pod_templates"] = serialize_pod_templates(
                self.context, self)
        if "annexes" in extra_include:
            result = serialize_extra_include_annexes(result, self)
        if "attendees" in extra_include:
            result["extra_include_attendees"] = serialize_attendees(
                self.context, extra_include_name="attendees", base_serializer=self)
        return result

    def _include_custom(self, obj, result):
        """Include "date" by default."""
        if self.fullobjects or \
           "date" in self.metadata_fields or \
           self.get_param('include_base_data', True):
            field = get_dx_field(obj, "date")
            # serialize the field
            serializer = queryMultiAdapter(
                (field, obj, self.request), IFieldSerializer
            )
            result["date"] = serializer()
        return result

    def _additional_values(self, result, additional_values):
        """ """
        # add some formatted values
        tool = api.portal.get_tool('portal_plonemeeting')
        # Products.PloneMeeting 4.1/4.2 compatibility
        if "*" in additional_values or "formatted_assembly" in additional_values:
            result["formatted_assembly"] = self.context.get_assembly(striked=True)
        if "*" in additional_values or "formatted_date" in additional_values:
            result["formatted_date"] = tool.format_date(
                self.context.date, short=True, with_hour=True)
        if "*" in additional_values or "formatted_date_short" in additional_values:
            result["formatted_date_short"] = tool.format_date(
                self.context.date, short=True, with_hour=False)
        if "*" in additional_values or "formatted_date_long" in additional_values:
            result["formatted_date_long"] = tool.format_date(
                self.context.date, short=False, with_hour=True)
        return result


@implementer(ISerializeToJson)
@adapter(IMeeting, Interface)
class SerializeToJson(BaseSerializeMeetingToJson, BaseDXSerializeFolderToJson):
    """ """


@implementer(ISerializeToJsonSummary)
@adapter(IMeeting, Interface)
class SerializeToJsonSummary(BaseSerializeMeetingToJson, PMBrainJSONSummarySerializer):
    """ """



# -*- coding: utf-8 -*-

from plone import api
from plonemeeting.restapi.config import CONFIG_ID_NOT_FOUND_ERROR
from plonemeeting.restapi.services.search import BaseSearchGet
from plonemeeting.restapi.utils import rest_uuid_to_object
from zExceptions import BadRequest


UID_REQUIRED_ERROR = 'The "UID or uid" parameter must be given!'
UID_WRONG_TYPE_ERROR = (
    "The element UID does not correspond to the type managed by this endpoint! "
    "Consider using @get endpoint or another specific endpoint!"
)


class UidSearchGet(BaseSearchGet):
    """Returns a serialized content object based on required UID parameter."""

    required_meta_type_id = None

    def __init__(self, context, request):
        super(UidSearchGet, self).__init__(context, request)
        self.external_id = self.request.form.get(
            "externalIdentifier",
            self.request.form.get("external_id"),
        )
        self.uid = self._uid
        self.config_id = self._config_id
        if self.config_id:
            self.tool = api.portal.get_tool("portal_plonemeeting")
            self.cfg = self.tool.get(self.config_id, None)
            if not self.cfg:
                raise BadRequest(CONFIG_ID_NOT_FOUND_ERROR % self.config_id)

    @property
    def _config_id(self):
        return super(UidSearchGet, self)._config_id

    @property
    def _uid(self):
        uid = self.request.form.get("UID") or self.request.form.get("uid")
        if not uid and not self.external_id:
            raise BadRequest(UID_REQUIRED_ERROR)
        return uid

    def _check_res_type(self, res):
        """ """
        if self.required_meta_type_id and res["items"]:
            # we have the portal_type in res, we need to get the meta_type
            # to compare it with self.required_meta_type_id
            portal_type = api.portal.get_tool("portal_types")[res["items"][0]["@type"]]
            # AT, meta_type is directly stored in content_meta_type
            if portal_type.__class__.__name__ == 'DynamicViewTypeInformation':
                meta_type = portal_type.content_meta_type
            else:
                # DX, klass is the dotted path the python klass
                meta_type = portal_type.klass.split('.')[-1]
            if self.required_meta_type_id != meta_type:
                raise BadRequest(UID_WRONG_TYPE_ERROR)

    def reply(self):
        """ """
        res = super(UidSearchGet, self).reply()
        self._check_res_type(res)
        if res["items"]:
            # we only have one single result
            res = res["items"][0]
        else:
            if self.uid:
                # will raise if element exist but inaccessible or not exist
                # do not try_restricted as it was just done in this endpoint
                rest_uuid_to_object(self.uid, try_restricted=False, in_name_of=self.in_name_of)
        return res


class ItemGet(UidSearchGet):
    """Get an item from it's UID, just check that returned value is a MeetingItem."""

    required_meta_type_id = "MeetingItem"


class MeetingGet(UidSearchGet):
    """Get a meeting from it's UID, just check that returned value is a Meeting."""

    required_meta_type_id = "Meeting"



# -*- coding: utf-8 -*-

from plone.app.textfield.interfaces import IRichText
from plone.dexterity.interfaces import IDexterityContent
from plone.restapi.deserializer import json_body
from plone.restapi.deserializer.dxfields import RichTextFieldDeserializer
from plone.restapi.interfaces import IFieldDeserializer
from plonemeeting.restapi.interfaces import IPMRestapiLayer
from plonemeeting.restapi.utils import clean_html
from zope.component import adapter
from zope.interface import implementer


@implementer(IFieldDeserializer)
@adapter(IRichText, IDexterityContent, IPMRestapiLayer)
class PMRichTextFieldDeserializer(RichTextFieldDeserializer):
    def __call__(self, value):
        value = super(PMRichTextFieldDeserializer, self).__call__(value)
        should_clean_html = json_body(self.request).get('clean_html', True)
        if should_clean_html and value:
            value._raw_holder.value = clean_html(value.raw)
        return value



# -*- coding: utf-8 -*-

from collective.documentgenerator.content.pod_template import IConfigurablePODTemplate
from imio.helpers.content import base_hasattr
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plonemeeting.restapi.serializer.base import BaseDXSerializeFolderToJson
from plonemeeting.restapi.serializer.summary import PMBrainJSONSummarySerializer
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class BaseSerializePodTemplateToJson(object):
    """ """

    def _include_custom(self, obj, result):
        """Include outputs (format/url)."""
        if base_hasattr(self, "original_context"):
            result["outputs"] = []
            original_context_url = self.original_context.absolute_url()
            url_pattern = "{0}/document-generation?template_uid={1}&output_format={2}"
            for output_format in self.context.get_available_formats():
                data = {}
                data["format"] = output_format
                data["url"] = url_pattern.format(
                    original_context_url, result["UID"], output_format)
                result["outputs"].append(data)
        return result


@implementer(ISerializeToJson)
@adapter(IConfigurablePODTemplate, Interface)
class SerializeToJson(BaseSerializePodTemplateToJson, BaseDXSerializeFolderToJson):
    """ """


@implementer(ISerializeToJsonSummary)
@adapter(IConfigurablePODTemplate, Interface)
class SerializeToJsonSummary(BaseSerializePodTemplateToJson, PMBrainJSONSummarySerializer):
    """ """



# -*- coding: utf-8 -*-

from imio.helpers.content import uuidsToObjects
from plone import api
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plonemeeting.restapi.serializer.base import BaseATSerializeFolderToJson
from plonemeeting.restapi.serializer.base import serialize_attendees
from plonemeeting.restapi.serializer.base import serialize_extra_include_annexes
from plonemeeting.restapi.serializer.base import serialize_pod_templates
from plonemeeting.restapi.serializer.summary import PMBrainJSONSummarySerializer
from plonemeeting.restapi.utils import build_catalog_query
from Products.PloneMeeting.interfaces import IMeetingItem
from Products.PloneMeeting.utils import get_internal_number
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class BaseSerializeItemToJson(object):
    """ """

    def _available_extra_includes(self, result):
        """ """
        result["@extra_includes"] = [
            "annexes",
            "attendees",
            "proposing_group",
            "category",
            "classifier",
            "groups_in_charge",
            "associated_groups",
            "pod_templates",
            "meeting",
            "linked_items",
            "*deliberation*",
            "config",
        ]
        return result

    def _extra_include_linked_items(self, result):
        """ """
        result["extra_include_linked_items"] = []
        # get the modes so we know which linked items to return
        modes = self.get_param('mode', ["auto"], extra_include_name="linked_items")
        linked_items = []
        for mode in modes:
            # every manually linked items
            if mode == "manual":
                linked_items += self.context.getManuallyLinkedItems(only_viewable=True)
            # every auto linked items, from first predecessor to last successor
            elif mode == "auto":
                # by default parameters include_successors=True, this will return
                # every auto linked items
                linked_items += self.context.get_predecessors(only_viewable=True)
            elif mode == "predecessor":
                predecessor = self.context.get_predecessor(unrestricted=False)
                if predecessor:
                    linked_items.append(predecessor)
            elif mode == "predecessors":
                linked_items += self.context.get_predecessors(
                    only_viewable=True, include_successors=False)
            elif mode == "successors":
                linked_items += self.context.get_successors(unrestricted=False)
            elif mode == "every_successors":
                linked_items += self.context.get_every_successors(unrestricted=False)
        query = build_catalog_query(self, extra_include_name="linked_items")
        if query:
            linked_item_uids = [linked_item.UID() for linked_item in linked_items]
            # we may unrestricted=True as we get linked items with unrestricted=False here above
            linked_items = uuidsToObjects(
                linked_item_uids, ordered=True, query=query, unrestricted=True)
        result["extra_include_linked_items"] = [
            self._get_serializer(linked_item, "linked_items")()
            for linked_item in linked_items]
        result["extra_include_linked_items_items_total"] = len(linked_items)
        return result

    def _extra_include(self, result):
        """To be simplified when moving MeetingItem to DX as extra_include name
           will be the same as attribute name defined in MeetingItem schema."""
        extra_include = self._get_asked_extra_include()
        if "proposing_group" in extra_include:
            proposing_group = self.context.getProposingGroup(theObject=True)
            result["extra_include_proposing_group"] = {}
            if proposing_group:
                serializer = self._get_serializer(proposing_group, "proposing_group")
                result["extra_include_proposing_group"] = serializer()
        if "category" in extra_include:
            category = self.context.getCategory(theObject=True, real=True)
            result["extra_include_category"] = {}
            if category:
                serializer = self._get_serializer(category, "category")
                result["extra_include_category"] = serializer()
        if "classifier" in extra_include:
            classifier = self.context.getClassifier(theObject=True)
            result["extra_include_classifier"] = {}
            if classifier:
                serializer = self._get_serializer(classifier, "classifier")
                result["extra_include_classifier"] = serializer()
        if "groups_in_charge" in extra_include:
            groups_in_charge = self.context.getGroupsInCharge(theObjects=True)
            result["extra_include_groups_in_charge"] = []
            for group_in_charge in groups_in_charge:
                serializer = self._get_serializer(group_in_charge, "groups_in_charge")
                result["extra_include_groups_in_charge"].append(serializer())
        if "associated_groups" in extra_include:
            associated_groups = self.context.getAssociatedGroups(theObjects=True)
            result["extra_include_associated_groups"] = []
            for associated_group in associated_groups:
                serializer = self._get_serializer(associated_group, "associated_groups")
                result["extra_include_associated_groups"].append(serializer())
        if "annexes" in extra_include:
            result = serialize_extra_include_annexes(result, self)
        if "pod_templates" in extra_include:
            result["extra_include_pod_templates"] = serialize_pod_templates(
                self.context, self)
        if "meeting" in extra_include:
            meeting = self.context.getMeeting()
            result["extra_include_meeting"] = {}
            if meeting:
                serializer = self._get_serializer(meeting, "meeting")
                result["extra_include_meeting"] = serializer()
        if "linked_items" in extra_include:
            result = self._extra_include_linked_items(result)
        if "config" in extra_include:
            tool = api.portal.get_tool('portal_plonemeeting')
            cfg = tool.getMeetingConfig(self.context)
            result["extra_include_config"] = {}
            if cfg:
                serializer = self._get_serializer(cfg, "config")
                result["extra_include_config"] = serializer()
        if "attendees" in extra_include:
            result["extra_include_attendees"] = serialize_attendees(
                self.context, extra_include_name="attendees", base_serializer=self)

        # various type of deliberation may be included
        # if we find a key containing "deliberation", we use it
        # add pass it to documentgenerator helper.deliberation_for_restapi
        delib_extra_includes = [ei for ei in extra_include
                                if "deliberation" in ei]
        if delib_extra_includes:
            # make the @@document-generation helper view available on self
            view = self.context.restrictedTraverse("document-generation")
            helper = view.get_generation_context_helper()
            deliberation = helper.deliberation_for_restapi(delib_extra_includes)
            result["extra_include_deliberation"] = deliberation

        return result

    def _additional_values(self, result, additional_values):
        """ """
        # add some formatted values
        if "*" in additional_values or "formatted_itemAssembly" in additional_values:
            result["formatted_itemAssembly"] = self.context.getItemAssembly(striked=True)
        if "*" in additional_values or "formatted_itemNumber" in additional_values:
            result["formatted_itemNumber"] = self.context.getItemNumber(for_display=True)

        # values including computed values
        # need to use "token/title" format
        # all_copyGroups
        if "*" in additional_values or "all_copyGroups" in additional_values:
            all_copyGroups = self.context.getAllCopyGroups(
                auto_real_plone_group_ids=True
            )
            vocab = self.context.getField('copyGroups').Vocabulary(self.context)
            all_copyGroups = [
                {"token": cg, "title": vocab.getValue(cg)}
                for cg in all_copyGroups]
            result["all_copyGroups"] = all_copyGroups
        # all_groupsInCharge
        # XXX to be removed when using PloneMeeting 4.2 as now
        # groupsInCharge are stored including "auto" groupsInCharge
        if "*" in additional_values or "all_groupsInCharge" in additional_values:
            all_groupsInCharge = self.context.getGroupsInCharge(includeAuto=True)
            vocab = self.context.getField('groupsInCharge').Vocabulary(self.context)
            all_groupsInCharge = [
                {"token": gic, "title": vocab.getValue(gic)}
                for gic in all_groupsInCharge]
            result["all_groupsInCharge"] = all_groupsInCharge
        return result


@implementer(ISerializeToJson)
@adapter(IMeetingItem, Interface)
class SerializeToJson(BaseSerializeItemToJson, BaseATSerializeFolderToJson):
    """ """

    def _include_fields(self, obj):
        """Manage internal_numbe manually as it is not a field for now but
           a stored attribute, will be a field when item will be DX."""
        result = super(SerializeToJson, self)._include_fields(obj)
        if self.fullobjects or "internal_number" in self.metadata_fields:
            result["internal_number"] = get_internal_number(obj)
        return result


@implementer(ISerializeToJsonSummary)
@adapter(IMeetingItem, Interface)
class SerializeToJsonSummary(BaseSerializeItemToJson, PMBrainJSONSummarySerializer):
    """ """



# -*- coding: utf-8 -*-

from datetime import datetime
from plonemeeting.restapi.services.attendees import FIRST_UID_TYPE
from plonemeeting.restapi.services.attendees import SECOND_UID_TYPE
from plonemeeting.restapi.services.attendees import URL_ATTENDEE_UID_REQUIRED_ERROR
from plonemeeting.restapi.services.attendees import URL_UID_REQUIRED_ERROR
from plonemeeting.restapi.services.attendees import WRONG_ATTENDEE_TYPE
from plonemeeting.restapi.tests.base import BaseTestCase
from plonemeeting.restapi.utils import UID_NOT_FOUND_ERROR
from Products.CMFCore.permissions import View
from Products.PloneMeeting.browser.itemattendee import WRONG_POSITION_TYPE
from Products.PloneMeeting.tests.PloneMeetingTestCase import DEFAULT_USER_PASSWORD

import transaction


class testServiceAttendees(BaseTestCase):
    """GET/POST @attendees endpoints on item and meeting."""

    def setUp(self):
        """ """
        super(testServiceAttendees, self).setUp()
        self._setUpOrderedContacts()
        self.changeUser("pmManager")
        # item out of meeting
        self.item0 = self.create("MeetingItem", proposingGroup=self.developers_uid)
        self.item0_uid = self.item0.UID()
        self.item0_url = self.item0.absolute_url()
        self.item1 = self.create("MeetingItem", proposingGroup=self.developers_uid)
        self.item1_uid = self.item1.UID()
        self.item1_url = self.item1.absolute_url()
        self.item2 = self.create("MeetingItem", proposingGroup=self.vendors_uid)
        self.item2_uid = self.item2.UID()
        self.item2_url = self.item2.absolute_url()
        self.meeting = self.create("Meeting", date=datetime(2021, 9, 23, 10, 0))
        self.meeting_uid = self.meeting.UID()
        self.meeting_url = self.meeting.absolute_url()
        self.presentItem(self.item1)
        self.presentItem(self.item2)
        # setUp absent/excused on meeting and item
        self.hp1_uid = self.meeting.get_attendees()[0]
        self.hp2_uid = self.meeting.get_attendees()[1]
        self.hp3_uid = self.meeting.get_attendees()[2]
        self.hp4_uid = self.meeting.get_attendees()[3]
        self.meeting.ordered_contacts[self.hp2_uid]['attendee'] = False
        self.meeting.ordered_contacts[self.hp2_uid]['excused'] = True
        # absent on item
        self.meeting.item_absents[self.item1_uid] = [self.hp3_uid]
        # non_attendee on item
        self.meeting.item_non_attendees[self.item1_uid] = [self.hp4_uid]
        self.meeting._p_changed = True
        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        transaction.abort()

    def test_restapi_get_attendees_endpoint(self):
        """The @attendees GET on meeting and item."""
        # check response, especially order and attendee_type
        # Meeting
        endpoint_url = "{0}/@attendees/{1}".format(
            self.portal_url, self.meeting_uid)
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json[0]['UID'], self.hp1_uid)
        self.assertEqual(json[0]['attendee_type'],
                         {u'token': u'present', u'title': u'Present'})
        self.assertEqual(json[0]['signatory'], u'1')
        self.assertEqual(json[1]['UID'], self.hp2_uid)
        self.assertEqual(json[1]['attendee_type'],
                         {u'token': u'excused', u'title': u'Absent (excused)'})
        self.assertEqual(json[1]['signatory'], None)
        self.assertEqual(json[2]['UID'], self.hp3_uid)
        self.assertEqual(json[2]['attendee_type'],
                         {u'token': u'present', u'title': u'Present'})
        self.assertEqual(json[2]['signatory'], None)
        self.assertEqual(json[3]['UID'], self.hp4_uid)
        self.assertEqual(json[3]['attendee_type'],
                         {u'token': u'present', u'title': u'Present'})
        self.assertEqual(json[3]['signatory'], u'2')
        # MeetingItem
        endpoint_url = "{0}/@attendees/{1}".format(
            self.portal_url, self.item1_uid)
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json[0]['UID'], self.hp1_uid)
        self.assertEqual(json[0]['attendee_type'],
                         {u'token': u'present', u'title': u'Present'})
        self.assertEqual(json[0]['signatory'], u'1')
        self.assertEqual(json[1]['UID'], self.hp2_uid)
        self.assertEqual(json[1]['attendee_type'],
                         {u'token': u'excused', u'title': u'Absent (excused)'})
        self.assertEqual(json[1]['signatory'], None)
        self.assertEqual(json[2]['UID'], self.hp3_uid)
        self.assertEqual(json[2]['attendee_type'],
                         {u'token': u'absent', u'title': u'Absent'})
        self.assertEqual(json[2]['signatory'], None)
        self.assertEqual(json[3]['UID'], self.hp4_uid)
        self.assertEqual(json[3]['attendee_type'],
                         {u'token': u'non_attendee', u'title': u'Non attendee'})
        # a non_attendee can not be signatory
        self.assertEqual(json[3]['signatory'], None)

        # item out of meeting
        endpoint_url = "{0}/@attendees/{1}".format(
            self.portal_url, self.item0_uid)
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json, [])

        # must be able to view the item to get attendees
        self.api_session.auth = ("pmCreator1", DEFAULT_USER_PASSWORD)
        self.changeUser("pmCreator1")
        self.assertTrue(self.hasPermission(View, self.item1))
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        self.api_session.auth = ("pmCreator2", DEFAULT_USER_PASSWORD)
        self.changeUser("pmCreator2")
        self.assertFalse(self.hasPermission(View, self.item1))
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 400, response.content)

    def test_restapi_get_and_patch_attendee_uids_required(self):
        """The meeting/item UID and attendee UID is required for @attendee
           endpoint GET and PATCH."""
        # no uids at all
        endpoint_url = "{0}/@attendee".format(self.portal_url)
        # GET
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': URL_UID_REQUIRED_ERROR, u'type': u'BadRequest'})
        # PATCH
        response = self.api_session.patch(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': URL_UID_REQUIRED_ERROR, u'type': u'BadRequest'})

        # attendee uid required
        endpoint_url = "{0}/@attendee/{1}".format(self.portal_url, self.meeting_uid)
        # GET
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': URL_ATTENDEE_UID_REQUIRED_ERROR, u'type': u'BadRequest'})
        # PATCH
        response = self.api_session.patch(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': URL_ATTENDEE_UID_REQUIRED_ERROR, u'type': u'BadRequest'})

        # first uid must be item or meeting
        endpoint_url = "{0}/@attendee/{1}/{2}".format(
            self.portal_url, self.hp1_uid, self.hp1_uid)
        # GET
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': FIRST_UID_TYPE, u'type': u'BadRequest'})
        # GET
        response = self.api_session.patch(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': FIRST_UID_TYPE, u'type': u'BadRequest'})

        # second uid must be held_position
        endpoint_url = "{0}/@attendee/{1}/{2}".format(
            self.portal_url, self.meeting_uid, self.meeting_uid)
        # GET
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': SECOND_UID_TYPE, u'type': u'BadRequest'})
        # PATCH
        response = self.api_session.patch(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': SECOND_UID_TYPE, u'type': u'BadRequest'})

        # first and second uid must exist
        # GET
        endpoint_url = "{0}/@attendee/{1}0/{2}".format(
            self.portal_url, self.meeting_uid, self.hp1_uid)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': UID_NOT_FOUND_ERROR % (self.meeting_uid + "0"),
             u'type': u'BadRequest'})
        endpoint_url = "{0}/@attendee/{1}/{2}0".format(
            self.portal_url, self.meeting_uid, self.hp1_uid)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': UID_NOT_FOUND_ERROR % (self.hp1_uid + "0"),
             u'type': u'BadRequest'})
        # PATCH
        endpoint_url = "{0}/@attendee/{1}0/{2}".format(
            self.portal_url, self.meeting_uid, self.hp1_uid)
        response = self.api_session.patch(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': UID_NOT_FOUND_ERROR % (self.meeting_uid + "0"),
             u'type': u'BadRequest'})
        endpoint_url = "{0}/@attendee/{1}/{2}0".format(
            self.portal_url, self.meeting_uid, self.hp1_uid)
        response = self.api_session.patch(endpoint_url)
        self.assertEqual(
            response.json(),
            {u'message': UID_NOT_FOUND_ERROR % (self.hp1_uid + "0"),
             u'type': u'BadRequest'})

    def test_restapi_get_attendee_endpoint(self):
        """The @attendee GET on meeting and item."""
        # check response, especially order and attendee_type
        data = {
            self.meeting_uid: {
                # hp_uid: (attendee_type, signatory)
                self.hp1_uid: ('present', '1'),
                self.hp2_uid: ('excused', None),
                self.hp3_uid: ('present', None),
                self.hp4_uid: ('present', '2'), },
            self.item1_uid: {
                self.hp1_uid: ('present', '1'),
                self.hp2_uid: ('excused', None),
                self.hp3_uid: ('absent', None),
                self.hp4_uid: ('non_attendee', None), },
            self.item2_uid: {
                self.hp1_uid: ('present', '1'),
                self.hp2_uid: ('excused', None),
                self.hp3_uid: ('present', None),
                self.hp4_uid: ('present', '2'), }, }
        for url in (self.meeting_uid, self.item1_uid, self.item2_uid):
            for hp_uid in (self.hp1_uid, self.hp2_uid, self.hp3_uid, self.hp4_uid):
                endpoint_url = "{0}/@attendee/{1}/{2}".format(
                    self.portal_url, url, hp_uid)
                response = self.api_session.get(endpoint_url)
                json = response.json()
                self.assertEqual(json['position_type'],
                                 {u'title': u'D\xe9faut', u'token': u'default'})
                self.assertEqual(json['UID'], hp_uid)
                self.assertEqual(json['attendee_type']['token'], data[url][hp_uid][0])
                self.assertEqual(json['signatory'], data[url][hp_uid][1])

    def test_restapi_patch_meeting_attendee_endpoint(self):
        """The @attendee PATCH on meeting."""
        # test an attendee that is present
        self.assertTrue(self.hp1_uid in self.meeting.get_attendees())
        # set it absent
        json = {"attendee_type": "absent", }
        endpoint_url = "{0}/@attendee/{1}/{2}".format(
            self.portal_url, self.meeting_uid, self.hp1_uid)
        response = self.api_session.patch(endpoint_url, json=json)
        # this will generate an error because self.hp1_uid is signatory
        self.assertEqual(response.status_code, 500, response.content)
        self.assertEqual(
            response.json(),
            {u'message': u'can_not_remove_attendee_defined_as_signatory',
             u'type': u'Invalid'})
        # set self.hp1_uid no more signatory so it may be set absent
        self.assertEqual(self.meeting.get_signatories()[self.hp1_uid], '1')
        json = {"signatory": 0, }
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.json()["attendee_type"],
                         {u'token': u'present', u'title': u'Present'})
        self.assertEqual(response.json()["signatory"], None)
        # this time self.hp1_uid me be set absent
        json = {"attendee_type": "absent", }
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.json()["attendee_type"],
                         {u'token': u'absent', u'title': u'Absent'})
        # set it excused
        json = {"attendee_type": "excused", }
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.json()["attendee_type"],
                         {u'token': u'excused', u'title': u'Absent (excused)'})
        # set it present
        json = {"attendee_type": "present", }
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.json()["attendee_type"],
                         {u'token': u'present', u'title': u'Present'})
        # will raise Unauthorized if user not able to edit meeting
        self.api_session.auth = ("pmCreator1", DEFAULT_USER_PASSWORD)
        json = {"attendee_type": "absent", }
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.status_code, 401, response.content)

    def test_restapi_patch_item_attendee_endpoint(self):
        """The @attendee PATCH on item."""
        # test an attendee that is excused on the meeting
        # not possible to change it's attendee_type on item
        self.assertTrue(self.hp2_uid in self.meeting.get_excused())
        # try to set it absent, will fail as not present on the meeting
        self.api_session.auth = ("pmManager", DEFAULT_USER_PASSWORD)
        json = {"attendee_type": "absent", }
        endpoint_url = "{0}/@attendee/{1}/{2}".format(
            self.portal_url, self.item1_uid, self.hp2_uid)
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.status_code, 400, response.content)
        self.assertEqual(
            response.json(),
            {u'message': u'Can not set "Absent" a person that is not present on the meeting!',
             u'type': u'BadRequest'})
        # same for non_attendee
        json = {"attendee_type": "non_attendee", }
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.status_code, 400, response.content)
        self.assertEqual(
            response.json(),
            {u'message': u'Can not set "Non attendee" a person that is not present on the meeting!',
             u'type': u'BadRequest'})
        # wrong attendee type
        json = {"attendee_type": "unknown", }
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.status_code, 400, response.content)
        self.assertEqual(
            response.json(),
            {u'message': WRONG_ATTENDEE_TYPE % 'unknown', u'type': u'BadRequest'})
        # signatory
        # trying to set absent as signatory will not work
        self.assertTrue(self.hp2_uid in self.meeting.get_excused())
        json = {"signatory": 1}
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.status_code, 400, response.content)
        self.assertEqual(
            response.json(),
            {u'message': u'Can not set "Signatory" a person that is not present on the meeting!',
             u'type': u'BadRequest'})
        # work with hp1_uid, already signatory on the meeting
        endpoint_url = "{0}/@attendee/{1}/{2}".format(
            self.portal_url, self.item1_uid, self.hp1_uid)
        self.assertTrue(self.hp1_uid in self.item1.get_item_signatories())
        # not able to set signatory for an item a user already signatory on the meeting
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.status_code, 400, response.content)
        self.assertEqual(
            response.json(),
            {u'message': u'Can not set "Signatory" a person that is already signatory on the meeting!',
             u'type': u'BadRequest'})

        # now with self.hp3_uid that is not signatory on the meeting not on the item
        endpoint_url = "{0}/@attendee/{1}/{2}".format(
            self.portal_url, self.item1_uid, self.hp3_uid)
        self.assertFalse(self.hp3_uid in self.item1.get_item_signatories())
        self.assertFalse(self.hp3_uid in self.item1.get_item_signatories(real=True))
        json = {"signatory": 1}
        response = self.api_session.patch(endpoint_url, json=json)
        transaction.commit()
        self.assertEqual(response.json()['signatory'], '1')
        # can change signatory even when already set
        json = {"signatory": 2}
        response = self.api_session.patch(endpoint_url, json=json)
        transaction.commit()
        self.assertEqual(response.json()['signatory'], '2')
        json = {"signatory": 3}
        response = self.api_session.patch(endpoint_url, json=json)
        transaction.commit()
        self.assertEqual(response.json()['signatory'], '3')
        self.assertTrue(self.hp3_uid in self.item1.get_item_signatories())
        self.assertTrue(self.hp3_uid in self.item1.get_item_signatories(real=True))
        # use signature number 0 to remove a signatory
        json = {"signatory": 0}
        response = self.api_session.patch(endpoint_url, json=json)
        transaction.commit()
        self.assertIsNone(response.json()['signatory'])
        # trying to remove a non signatory will do nothing
        json = {"signatory": 0}
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.status_code, 200, response.content)
        self.assertIsNone(response.json()['signatory'])
        # redefine signatory and position_type
        # wrong position_type
        json = {"signatory": 1, "position_type": 'unknown'}
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(response.status_code, 400, response.content)
        self.assertEqual(
            response.json(),
            {u'message': WRONG_POSITION_TYPE % "unknown",
             u'type': u'BadRequest'})
        json = {"signatory": 2, "position_type": 'administrator'}
        response = self.api_session.patch(endpoint_url, json=json)
        # change signature_number and signatory position_type
        response_json = response.json()
        self.assertEqual(response_json['signatory'], u'2')
        self.assertEqual(response_json['signatory_position_type'],
                         {u'title': u'Administratrice',
                          u'token': u'administrator'})

    def test_restapi_patch_item_attendee_position_type_endpoint(self):
        """The @attendee PATCH on item that changes attendee position_type."""
        # test an attendee that is excused on the meeting
        # not possible to change it's position_type on item
        self.assertTrue(self.hp2_uid in self.meeting.get_excused())
        # try to set it absent, will fail as not present on the meeting
        self.api_session.auth = ("pmManager", DEFAULT_USER_PASSWORD)
        json = {"position_type": "administrator", }
        endpoint_url = "{0}/@attendee/{1}/{2}".format(
            self.portal_url, self.item1_uid, self.hp2_uid)
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(
            response.json()['position_type'],
            {u'title': u'Administrateur',
             u'token': u'administrator'})
        # can change position_type that was already changed
        json = {"position_type": "default", }
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(
            response.json()['position_type'],
            {u'title': u'D\xe9faut', u'token': u'default'})
        # remove redefined position_type
        json = {"position_type": "administrator", }
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(
            response.json()['position_type'],
            {u'title': u'Administrateur',
             u'token': u'administrator'})
        json = {"position_type": "administrator", "remove": 1}
        response = self.api_session.patch(endpoint_url, json=json)
        self.assertEqual(
            response.json()['position_type'],
            {u'title': u'D\xe9faut', u'token': u'default'})

    def test_restapi_extra_include_attendees(self):
        """extra_include=attendees is available on item and meeting."""
        self.api_session.auth = ("pmManager", DEFAULT_USER_PASSWORD)
        # Meeting
        endpoint_url = "{0}/@get?UID={1}&extra_include=attendees".format(
            self.portal_url, self.meeting_uid)
        response = self.api_session.get(endpoint_url)
        response_json = response.json()
        self.assertEqual(
            response_json['extra_include_attendees'][0]['attendee_type'],
            {u'title': u'Present', u'token': u'present'})
        self.assertEqual(
            response_json['extra_include_attendees'][0]['signatory'], '1')
        self.assertEqual(
            response_json['extra_include_attendees'][1]['attendee_type'],
            {u'title': u'Absent (excused)', u'token': u'excused'},)
        self.assertIsNone(response_json['extra_include_attendees'][1]['signatory'])
        self.assertEqual(
            response_json['extra_include_attendees'][3]['attendee_type'],
            {u'title': u'Present', u'token': u'present'})
        self.assertEqual(
            response_json['extra_include_attendees'][3]['signatory'], '2')
        # Item
        endpoint_url = "{0}/@get?UID={1}&extra_include=attendees".format(
            self.portal_url, self.item1_uid)
        response = self.api_session.get(endpoint_url)
        response_json = response.json()
        self.assertEqual(
            response_json['extra_include_attendees'][0]['attendee_type'],
            {u'title': u'Present', u'token': u'present'})
        self.assertEqual(
            response_json['extra_include_attendees'][0]['signatory'], '1')
        self.assertEqual(
            response_json['extra_include_attendees'][1]['attendee_type'],
            {u'title': u'Absent (excused)', u'token': u'excused'},)
        self.assertIsNone(response_json['extra_include_attendees'][1]['signatory'])
        self.assertEqual(
            response_json['extra_include_attendees'][3]['attendee_type'],
            {u'token': u'non_attendee', u'title': u'Non attendee'})
        self.assertIsNone(response_json['extra_include_attendees'][3]['signatory'])
        # item out of meeting
        endpoint_url = "{0}/@get?UID={1}&extra_include=attendees".format(
            self.portal_url, self.item0_uid)
        response = self.api_session.get(endpoint_url)
        response_json = response.json()
        self.assertEqual(response_json['extra_include_attendees'], [])


def test_suite():
    from unittest import TestSuite, makeSuite

    suite = TestSuite()
    # add a prefix to avoid every PM tests to be executed
    suite.addTest(makeSuite(testServiceAttendees, prefix="test_restapi_"))
    return suite



# -*- coding: utf-8 -*-

CONFIG_ID_ERROR = 'The "config_id" parameter must be given!'
CONFIG_ID_TYPE_ERROR = 'A "config_id" can not be given when parameter "type" is "config"!'
CONFIG_ID_NOT_FOUND_ERROR = 'The given "config_id" named "%s" was not found!'
ANNEXES_FILTER_VALUES = [
    "to_print", "confidential", "publishable", "to_sign", "signed"]
INDEX_CORRESPONDENCES = {
    'getConfigId': 'config_id',
    'review_state': 'state',
    'portal_type': 'type',
    'UID': 'uid',
    'externalIdentifier': 'external_id',
}



# -*- coding: utf-8 -*-

from plone import api
from plonemeeting.restapi.config import CONFIG_ID_NOT_FOUND_ERROR
from plonemeeting.restapi.services.search import BaseSearchGet
from zExceptions import BadRequest


class ConfigSearchGet(BaseSearchGet):
    """Returns a serialized content object.
    """

    config_id_optional = False

    def __init__(self, context, request):
        super(ConfigSearchGet, self).__init__(context, request)
        # we may ask for every configs by using config_id=*
        if self.config_id != "*":
            self.tool = api.portal.get_tool("portal_plonemeeting")
            self.cfg = self.tool.get(self.config_id, None)
            if not self.cfg:
                raise BadRequest(CONFIG_ID_NOT_FOUND_ERROR % self.config_id)

    @property
    def _type(self):
        # force to MeetingConfig
        return "MeetingConfig"

    def _set_query_before_hook(self):
        """ """
        query = super(ConfigSearchGet, self)._set_query_before_hook()
        query.pop('getConfigId')
        query['portal_type'] = self.type
        if self.config_id != "*":
            query['id'] = self.cfg.id
        else:
            query['sort_on'] = self.request.form.get('sort_on', 'getId')
        # by default only return active MeetingConfigs excepted if aksed
        if "review_state" not in query:
            query["review_state"] = "active"
        return query

    def reply(self):
        res = super(ConfigSearchGet, self).reply()
        if res["items"] and hasattr(self, "cfg"):
            # we asked for one single MeetingConfig, return it (not a list of result)
            res = res["items"][0]
        return res



# -*- coding: utf-8 -*-

from plone.restapi.deserializer import json_body
from plone.restapi.deserializer.atfields import DefaultFieldDeserializer
from plone.restapi.interfaces import IFieldDeserializer
from plonemeeting.restapi.utils import clean_html
from Products.Archetypes.interfaces import IBaseObject
from Products.Archetypes.interfaces.field import ITextField
from zope.component import adapter
from zope.interface import implementer
from zope.publisher.interfaces.browser import IBrowserRequest


@implementer(IFieldDeserializer)
@adapter(ITextField, IBaseObject, IBrowserRequest)
class TextFieldDeserializer(DefaultFieldDeserializer):
    def __call__(self, value):
        kwargs = {}
        should_clean_html = json_body(self.request).get('clean_html', True)
        if should_clean_html and value:
            value = clean_html(value)
        return value, kwargs



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from BeautifulSoup import BeautifulSoup
from HTMLParser import HTMLParser
from imio.helpers.content import uuidToObject
from lxml.html.clean import Cleaner
from plone import api
from plone.restapi.deserializer import boolean_value
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plonemeeting.restapi.config import INDEX_CORRESPONDENCES
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import _checkPermission
from Products.CMFPlone.utils import safe_unicode
from Products.PloneMeeting.config import MEETINGMANAGERS_GROUP_SUFFIX
from Products.PloneMeeting.utils import convert2xhtml
from zExceptions import BadRequest
from zope.component import queryMultiAdapter
from zope.globalrequest import getRequest


IN_NAME_OF_CONFIG_ID_UNAUTHORIZED = 'User "%s" must be Manager/MeetingManager for ' \
    'config "%s" to use "in_name_of=%s" option!'
IN_NAME_OF_UNAUTHORIZED = 'User "%s" must be Manager/MeetingManager to use "in_name_of=%s" option!'
IN_NAME_OF_USER_NOT_FOUND = 'The in_name_of user "%s" was not found!'
UID_NOT_ACCESSIBLE_ERROR = 'Element with UID "%s" was found in config "%s" but ' \
    'user "%s" can not access it!'
UID_NOT_ACCESSIBLE_IN_NAME_OF_ERROR = 'Element with UID "%s" was found in config ' \
    '"%s" but user "%s" can not access it (using "in_name_of" original power user "%s")!'
UID_NOT_FOUND_ERROR = 'No element found with UID "%s"!'


def check_in_name_of(cfg_id, data):
    """ """
    in_name_of = data.get("in_name_of", None)
    access_cfg_ids = None
    if in_name_of is not None:
        access_cfg_ids = get_poweraccess_configs()
        # if user not a (Meeting)Manager or a cfg_id is given and user is not
        # MeetingManager for it, raise Unauthorized
        if not access_cfg_ids:
            raise Unauthorized(IN_NAME_OF_UNAUTHORIZED %
                               (api.user.get_current().getId(), in_name_of))
        elif cfg_id and cfg_id not in access_cfg_ids:
            # not MeetingManager for the given cfg_id
            raise Unauthorized(IN_NAME_OF_CONFIG_ID_UNAUTHORIZED %
                               (api.user.get_current().getId(), cfg_id, in_name_of))
        user = api.user.get(in_name_of)
        if not user:
            raise BadRequest(IN_NAME_OF_USER_NOT_FOUND % in_name_of)
    return in_name_of, access_cfg_ids


def get_serializer(obj, extra_include_name=None, serializer=None, interface=ISerializeToJsonSummary):
    """ """
    request = getRequest()
    # check if need to use ISerializeToJson interface
    if interface != ISerializeToJson:
        prefix = ''
        if extra_include_name:
            prefix = "extra_include_{0}_".format(extra_include_name)
        if use_obj_serializer(request.form, prefix=prefix):
            interface = ISerializeToJson
    serializer = queryMultiAdapter((obj, request), interface)
    if extra_include_name:
        serializer._extra_include_name = extra_include_name
    return serializer


def get_param(value, default=False, extra_include_name=None, serializer=None):
    """If current serialized element is an extra_include,
       infos in request.form are relative to extra_include,
       else information are directly available.
       For extra_include, a parameter is passed like :
       ?extra_include=extra_include_name:parameter_name:value so
       ?extra_include_category_fullobjects or ?extra_include_category_metadata_fields=acronym."""
    request = getRequest()
    # extra_include_name is stored on serializer or passed as parameter when serializer
    # still not initialized, this is the case for parameter "fullobjects" as from this
    # will depend the interface to use to get the serializer
    extra_include_name = serializer and \
        getattr(serializer, "_extra_include_name", extra_include_name)
    if extra_include_name:
        # change param value
        value = "extra_include_{0}_{1}".format(extra_include_name, value)

    param = request.form.get(value, None)

    # param was not found in request.form
    if param is None:
        param = default
    elif default in (True, False):
        param = boolean_value(param)
    # if default is a list, then make sure we return a list
    elif isinstance(default, (tuple, list)) and not isinstance(param, (tuple, list)):
        param = [param]
    return param


def clean_html(value):
    """ """
    if clean_html and value:
        # we need a surrounding <p></p> or the content is not generated by appy.pod
        if not value.startswith(u'<p>') or not value.endswith(u'</p>'):
            value = u'<p>%s</p>' % value
        soup = BeautifulSoup(safe_unicode(value))
        soup_contents = soup.renderContents()
        if not isinstance(soup_contents, unicode):
            soup_contents = safe_unicode(soup_contents)
        # clean HTML with HTMLParser, it will remove special entities like &#xa0;
        soup_contents = HTMLParser().unescape(soup_contents)
        # clean HTML with lxml Cleaner
        cleaner = Cleaner()
        soup_contents = cleaner.clean_html(soup_contents)
        # clean_html surrounds the cleaned HTML with <div>...</div>... removes it!
        if soup_contents.startswith(u'<div>') and soup_contents.endswith(u'</div>'):
            soup_contents = soup_contents[5:-6]
        if not soup_contents == value:
            value = soup_contents
    return value


def handle_html(obj, data):
    """ """
    return convert2xhtml(obj,
                         data,
                         image_src_to_data=True,
                         anonymize=True,
                         use_appy_pod_preprocessor=True)


def get_poweraccess_configs():
    '''
      Return the MeetingConfig ids the current user can access,
      so the active configs for which user is MeetingManager or every
      if user is Manager.
      This will be used to protect access to some config endpoints or
      functionnalities like "in_name_of".
    '''
    tool = api.portal.get_tool('portal_plonemeeting')
    if _checkPermission(ManagePortal, tool):
        cfg_ids = [cfg.id for cfg in tool.getActiveConfigs(
            check_using_groups=False, check_access=False)]
    else:
        cfg_ids = [
            group_id.split('_')[0] for group_id in
            tool.get_filtered_plone_groups_for_user(
                suffixes=[MEETINGMANAGERS_GROUP_SUFFIX])]
    return cfg_ids


def use_obj_serializer(form, prefix=''):
    """Methods that will determinate if regarding values in the request,
       the ISerializeToJson serializer should be used."""
    # boolean_value of "" is True
    return bool(
        boolean_value(form.get(prefix + 'fullobjects', False)) or
        [v for v in form if v.startswith(prefix + 'include_')] or
        form.get(prefix + "extra_include", []) or
        form.get(prefix + "metadata_fields", []) or
        form.get(prefix + "additional_values", []))


def rest_uuid_to_object(uid, try_restricted=True, in_name_of=None):
    """Return the object corresponding to given p_uid but manage cases when
       it was not found or is not available to current user."""
    obj = None
    if try_restricted:
        obj = uuidToObject(uid)
    if obj is None:
        # try to get it unrestricted
        obj = uuidToObject(uid, unrestricted=True)
        if obj:
            tool = api.portal.get_tool('portal_plonemeeting')
            cfg = tool.getMeetingConfig(obj)
            if in_name_of:
                msg = UID_NOT_ACCESSIBLE_IN_NAME_OF_ERROR % (
                    uid, cfg.getId(), in_name_of, api.user.get_current().getId())
            else:
                msg = UID_NOT_ACCESSIBLE_ERROR % (
                    uid, cfg.getId(), in_name_of or api.user.get_current().getId())
            raise BadRequest(msg)
        else:
            raise BadRequest(UID_NOT_FOUND_ERROR % uid)
    return obj


def build_catalog_query(serializer, extra_include_name=None):
    """ """

    # filters may be any portal_catalog index
    catalog = api.portal.get_tool('portal_catalog')
    query = {}
    for index in catalog.Indexes:
        value = serializer.get_param(
            index, None, extra_include_name=extra_include_name)
        if value is None:
            # try to get an index by it's correspondance name
            easy_index = INDEX_CORRESPONDENCES.get(index, None)
            if easy_index is not None:
                value = serializer.get_param(
                    easy_index, None, extra_include_name=extra_include_name)
        if value is not None:
            query[index] = value
    return query



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from imio.helpers.content import get_vocab
from imio.helpers.content import uuidToObject
from plone import api
from plone.restapi.deserializer import json_body
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.services import Service
from plonemeeting.restapi.serializer.base import serialize_attendees
from plonemeeting.restapi.utils import rest_uuid_to_object
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.utils import _checkPermission
from Products.CMFPlone.utils import safe_unicode
from Products.PloneMeeting.content.meeting import _validate_attendees_removed_and_order
from Products.PloneMeeting.content.meeting import _validate_attendees_signatories
from Products.PloneMeeting.interfaces import IMeetingContent
from Products.PloneMeeting.utils import _itemNumber_to_storedItemNumber
from zExceptions import BadRequest
from zope.component import adapter
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.interfaces import IPublishTraverse
from zope.publisher.interfaces import NotFound


FIRST_UID_TYPE = "The first provided UID must be the UID of an item or a meeting!"
SECOND_UID_TYPE = "The second provided UID must be the UID of an attendee (held_position)!"
URL_UID_REQUIRED_ERROR = "The object UID must be provided in the URL!"
URL_ATTENDEE_UID_REQUIRED_ERROR = "The attendee UID must be provided in the URL!"
WRONG_ATTENDEE_TYPE = "Wrong attendee_type : \"%s\""
WRONG_PARAMETERS = "Your request is not supported, please check parameters!"
WRONG_POSITION_TYPE = "Wrong position_type : \"%s\""


@implementer(IExpandableElement)
@adapter(IMeetingContent, Interface)
class Attendees(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, expand=False):
        result = {"attendees": {"@id": "{}/@attendees".format(self.context.absolute_url())}}
        if not expand:
            return result

        # extend batch? DEFAULT_BATCH_SIZE = 25
        # self.request.form['b_size'] = 50

        result = serialize_attendees(self.context)
        return result


@implementer(IPublishTraverse)
class AttendeesGet(Service):

    def __init__(self, context, request):
        super(AttendeesGet, self).__init__(context, request)
        self.uid = None
        self.tool = api.portal.get_tool("portal_plonemeeting")

    def publishTraverse(self, request, name):
        if self.uid is None:
            self.uid = name
        else:
            raise NotFound(self, name, request)
        return self

    def reply(self):

        # initialize self.context
        if self.uid is None:
            raise BadRequest("UID_REQUIRED_ERROR")
        json = json_body(self.request)
        self.context = rest_uuid_to_object(self.uid, in_name_of=json.get("in_name_of"))

        # process
        attendees = Attendees(self.context, self.request)
        return attendees(expand=True)


@implementer(IPublishTraverse)
class AttendeeGet(Service):

    def __init__(self, context, request):
        super(AttendeeGet, self).__init__(context, request)
        self.uid = None
        self.attendee_uid = None

    def publishTraverse(self, request, name):
        if self.uid is None:
            self.uid = name
        else:
            if self.attendee_uid is None:
                self.attendee_uid = name
            else:
                raise NotFound(self, name, request)
        return self

    def _init(self):
        """ """
        if self.uid is None:
            raise BadRequest(URL_UID_REQUIRED_ERROR)
        if self.attendee_uid is None:
            raise BadRequest(URL_ATTENDEE_UID_REQUIRED_ERROR)

        json = json_body(self.request)
        self.context = rest_uuid_to_object(self.uid, in_name_of=json.get("in_name_of"))
        if not self.context.getTagName() in ("MeetingItem", "Meeting", ):
            raise BadRequest(FIRST_UID_TYPE)
        self.attendee = rest_uuid_to_object(self.attendee_uid, in_name_of=json.get("in_name_of"))
        if self.attendee.portal_type not in ("held_position", ):
            raise BadRequest(SECOND_UID_TYPE)

    def reply(self):
        # check required data and initialize context
        self._init()

        # process
        result = serialize_attendees(self.context, attendee=self.attendee)
        return result and result[0]


@implementer(IPublishTraverse)
class AttendeePatch(AttendeeGet):

    def reply(self):

        # check required data and initialize context
        self._init()

        # process
        is_meeting = self.context.__class__.__name__ == "Meeting"
        if is_meeting and not _checkPermission(ModifyPortalContent, self.context):
            raise Unauthorized
        meeting = self.context if is_meeting else self.context.getMeeting()
        json = json_body(self.request)
        was_managed = False
        if "attendee_type" in json:
            self._manage_attendee_type(json, is_meeting, meeting)
            was_managed = True
        elif "signatory" in json:
            self._manage_signatory(json, is_meeting, meeting)
            was_managed = True
        elif "position_type" in json and not is_meeting:
            self._manage_position_type(json)
            was_managed = True
        else:
            raise BadRequest(WRONG_PARAMETERS)

        result = None
        if was_managed:
            result = serialize_attendees(self.context, attendee=self.attendee)
        return result and result[0]

    def _manage_attendee_type(self, json, is_meeting, meeting):
        """Change attendee type on Meeting or MeetingItem.
           So a present is set absent, an absent is set excused, ..."""
        attendee_type_mappings = {'present': '@@item_welcome_attendee_form',
                                  'absent': '@@item_byebye_attendee_form',
                                  'excused': '@@item_byebye_attendee_form',
                                  'non_attendee': '@@item_byebye_nonattendee_form',
                                  'attendee': None}

        attendee_type = json.get('attendee_type')
        if attendee_type not in attendee_type_mappings:
            raise BadRequest(WRONG_ATTENDEE_TYPE % attendee_type)

        # Meeting
        if is_meeting:
            if attendee_type == 'present':
                attendee_type = 'attendee'
            # call validation
            all_meeting_attendees = self.context.get_all_attendees()
            meeting_attendees = list(all_meeting_attendees)
            if attendee_type != 'attendee':
                meeting_attendees.remove(self.attendee_uid)
            _validate_attendees_removed_and_order(
                self.context,
                meeting_attendees,
                all_meeting_attendees,
                self.context.get_signatories())
            self.context._update_attendee_type(
                self.attendee_uid, attendee_type, force_clear=True)
        else:
            # MeetingItem
            form_name = attendee_type_mappings.get(attendee_type)
            self.request.form['person_uid'] = safe_unicode(self.attendee_uid)
            form = self.context.restrictedTraverse(form_name)
            form.update()
            form.person_uid = self.attendee_uid
            form.not_present_type = attendee_type
            form.apply_until_item_number = _itemNumber_to_storedItemNumber(
                str(json.get('until_item_number', u'0')))
            form.meeting = meeting
            error = form._doApply()
            if error:
                # we get a message in error, translate it
                msg = translate(error, context=self.request)
                raise BadRequest(msg)

    def _manage_signatory(self, json, is_meeting, meeting):
        """Manage signatories on Meeting and MeetingItem."""
        signature_number = json.get('signatory')
        if not isinstance(signature_number, int) or signature_number < 0 or signature_number > 20:
            raise BadRequest("Wrong signatory : \"%s\".  "
                             "An integer between 0 and 20 is required.  "
                             "Using 0 will unset signatory." % signature_number)
        signature_number = None if signature_number == 0 else safe_unicode(str(signature_number))
        # Meeting
        if is_meeting:
            # turn signature number integer into None or a unicode
            signature_numbers = self.context.get_signatories()
            signature_numbers.update({self.attendee_uid: signature_number})
            _validate_attendees_signatories(self.context, signature_numbers.values())
            self.context._update_signature_number(
                self.attendee_uid, signature_number or None)
        else:
            # MeetingItem
            self.request.form['person_uid'] = safe_unicode(self.attendee_uid)
            form_name = "@@item_redefine_signatory_form"
            if signature_number is None:
                form_name = "@@item_remove_redefined_signatory_form"
            form = self.context.restrictedTraverse(form_name)
            form.update()
            form.person_uid = self.attendee_uid
            if signature_number is not None:
                form.signature_number = signature_number
                # taken from json or left untouched
                position_type = json.get(
                    'position_type', uuidToObject(self.attendee_uid).position_type)
                form.position_type = position_type
            form.apply_until_item_number = _itemNumber_to_storedItemNumber(
                str(json.get('until_item_number', u'0')))
            form.meeting = meeting
            error = form._doApply()
            if error:
                # we get a message in error, translate it
                msg = translate(error, context=self.request)
                raise BadRequest(msg)

    def _manage_position_type(self, json):
        """Manage changing a position_type for an attendee on MeetingItem."""
        position_type = json.get('position_type')
        remove = json.get('remove', False)
        attendee = uuidToObject(self.attendee_uid)
        position_type_vocab = get_vocab(attendee, "PMPositionTypes")
        if position_type not in position_type_vocab:
            raise BadRequest(WRONG_POSITION_TYPE % position_type)

        self.request.form['person_uid'] = safe_unicode(self.attendee_uid)
        form_name = "@@item_remove_redefined_attendee_position_form" if remove \
            else "@@item_redefine_attendee_position_form"
        form = self.context.restrictedTraverse(form_name)
        form.update()
        form.person_uid = self.attendee_uid
        form.position_type = position_type
        form.apply_until_item_number = _itemNumber_to_storedItemNumber(
            str(json.get('until_item_number', u'0')))
        form.meeting = self.context.getMeeting()
        error = form._doApply()
        if error:
            # we get a message in error, translate it
            msg = translate(error, context=self.request)
            raise BadRequest(msg)



# -*- coding: utf-8 -*-

from imio.helpers.content import get_schema_fields
from plone.dexterity.interfaces import IDexterityContent
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.serializer.converters import json_compatible
from plone.restapi.serializer.dxfields import ChoiceFieldSerializer
from plone.restapi.serializer.dxfields import CollectionFieldSerializer
from plonemeeting.restapi import logger
from plonemeeting.restapi.interfaces import IPMRestapiLayer
from zope.component import adapter
from zope.interface import implementer
from zope.schema.interfaces import IChoice
from zope.schema.interfaces import ICollection
from zope.schema.interfaces import IField
from zope.schema.interfaces import IVocabularyTokenized
from zope.schema.vocabulary import SimpleTerm


@adapter(IChoice, IDexterityContent, IPMRestapiLayer)
@implementer(IFieldSerializer)
class PMChoiceFieldSerializer(ChoiceFieldSerializer):
    """Override to take elephantvocabulary into account."""

    def __call__(self):
        # Binding is necessary for named vocabularies
        if IField.providedBy(self.field):
            self.field = self.field.bind(self.context)
        value = self.get_value()
        # XXX with elephanvocabulary, "real" vocab is stored on vocabulary.vocab
        vocab = getattr(self.field.vocabulary, "vocab", self.field.vocabulary)
        if value is not None and IVocabularyTokenized.providedBy(vocab):
            try:
                term = self.field.vocabulary.getTerm(value)
            # If not found, get it from the z3c.form that maybe uses a MissingTerms adapter...
            except LookupError:
                try:
                    view = self.context.restrictedTraverse("@@view")
                    view.update()
                    # widget_name is the field name prefixed with behavior if any
                    widget_name = [field for field in get_schema_fields(self.context, prefix=True)
                                   if field[1].__name__ == self.field.__name__][0][0]
                    widget = view.widgets[widget_name]
                    term = widget.terms.getTerm(value)
                except Exception:
                    # at worse use value as title
                    term = SimpleTerm(value, title=value)
            finally:
                value = {"token": term.token, "title": term.title}

        return json_compatible(value)


@adapter(ICollection, IDexterityContent, IPMRestapiLayer)
@implementer(IFieldSerializer)
class PMCollectionFieldSerializer(CollectionFieldSerializer):
    """Override to take elephantvocabulary into account."""

    def __call__(self):
        # Binding is necessary for named vocabularies
        if IField.providedBy(self.field):
            self.field = self.field.bind(self.context)
        value = self.get_value()
        value_type = self.field.value_type
        if value is not None and IChoice.providedBy(value_type):
            # XXX with elephanvocabulary, "real" vocab is stored on vocabulary.vocab
            vocab = getattr(value_type.vocabulary, "vocab", value_type.vocabulary)
            if IVocabularyTokenized.providedBy(vocab):
                values = []
                for v in value:
                    try:
                        term = value_type.vocabulary.getTerm(v)
                        values.append({u"token": term.token, u"title": term.title})
                    except LookupError:
                        logger.warning("Term lookup error: %r" % v)
                value = values
        return json_compatible(value)



# -*- coding: utf-8 -*-

from plonemeeting.restapi.config import CONFIG_ID_ERROR
from plonemeeting.restapi.config import CONFIG_ID_NOT_FOUND_ERROR
from plonemeeting.restapi.tests.base import BaseTestCase

import transaction


class testServiceConfig(BaseTestCase):
    """ """

    def tearDown(self):
        self.api_session.close()

    def test_restapi_config_required_params(self):
        """ """
        endpoint_url = "{0}/@config".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.json(), {u"message": CONFIG_ID_ERROR, u"type": u"BadRequest"}
        )
        endpoint_url += "?config_id={0}".format(self.meetingConfig.getId())
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)

    def test_restapi_config_config_id_not_found(self):
        """ """
        endpoint_url = "{0}/@config?config_id=unknown".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.json(),
            {u"message": CONFIG_ID_NOT_FOUND_ERROR % "unknown", u"type": u"BadRequest"},
        )

    def test_restapi_config_endpoint(self):
        """@config"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        endpoint_url = "{0}/@config?config_id={1}".format(self.portal_url, cfg_id)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        # by default, no "items"
        self.assertFalse("items" in json)
        self.assertEqual(json["id"], cfg_id)
        self.assertEqual(json["UID"], cfg.UID())

    def test_restapi_config_every_configs_endpoint(self):
        """@config when config_id=*"""
        cfg = self.meetingConfig
        cfg2 = self.meetingConfig2
        endpoint_url = "{0}/@config?config_id=*".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["items_total"], 2)
        # configs are returned alphabetically
        cfg_titles = sorted((cfg.Title(), cfg2.Title()))
        self.assertEqual(json["items"][0]["title"], cfg_titles[0])
        self.assertEqual(json["items"][1]["title"], cfg_titles[1])
        # by default, only active configs are returned
        self.changeUser('siteadmin')
        self.do(cfg, 'deactivate')
        transaction.commit()
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["items_total"], 1)
        self.assertEqual(json["items"][0]["title"], cfg2.Title())
        # as this endpoint is a SearchGet we can pass any index
        # so we can get inactive MeetingConfigs
        endpoint_url += "&state=inactive"
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["items_total"], 1)
        self.assertEqual(json["items"][0]["title"], cfg.Title())

    def test_restapi_config_extra_include_categories(self):
        """@config"""
        # cfg2 uses disabled categories
        cfg = self.meetingConfig2
        self._enableField('category')
        cfg_id = cfg.getId()
        transaction.commit()
        # categories
        endpoint_url_pattern = "{0}/@config?config_id={1}&extra_include={2}"
        endpoint_url = endpoint_url_pattern.format(self.portal_url, cfg_id, "categories")
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        categories = json["extra_include_categories"]
        self.assertEqual(categories[0]["id"], u"deployment")
        self.assertEqual(categories[0]["@type"], u"meetingcategory")
        self.assertEqual(categories[0]["enabled"], True)
        self.assertEqual(categories[0]["review_state"], None)
        # disabled categories are also returned
        self.assertEqual(categories[-2]["id"], u"marketing")
        self.assertEqual(categories[-2]["@type"], u"meetingcategory")
        self.assertEqual(categories[-2]["enabled"], False)
        self.assertEqual(categories[-2]["review_state"], None)

    def test_restapi_config_extra_include_classifiers(self):
        """@config"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        endpoint_url_pattern = "{0}/@config?config_id={1}&extra_include={2}"
        endpoint_url = endpoint_url_pattern.format(self.portal_url, cfg_id, "classifiers")
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        classifiers = json["extra_include_classifiers"]
        self.assertEqual(len(classifiers), 3)
        self.assertEqual(classifiers[0]["id"], u"classifier1")
        self.assertEqual(classifiers[0]["@type"], u"meetingcategory")
        self.assertEqual(classifiers[0]["enabled"], True)
        self.assertEqual(classifiers[0]["review_state"], None)
        self.assertEqual(classifiers[1]["id"], u"classifier2")
        self.assertEqual(classifiers[2]["id"], u"classifier3")

    def test_restapi_config_extra_include_pod_templates(self):
        """@config"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        endpoint_url_pattern = "{0}/@config?config_id={1}&extra_include={2}"
        endpoint_url = endpoint_url_pattern.format(
            self.portal_url, cfg_id, "pod_templates"
        )
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        pod_templates = json["extra_include_pod_templates"]
        self.assertEqual(pod_templates[0]["id"], u"styles1")
        self.assertEqual(pod_templates[0]["@type"], u"StyleTemplate")
        self.assertEqual(pod_templates[4]["id"], u"itemTemplate")
        self.assertEqual(pod_templates[4]["@type"], u"ConfigurablePODTemplate")
        itemTemplate = cfg.podtemplates.itemTemplate
        self.assertEqual(len(pod_templates), 7)
        self.assertTrue(itemTemplate.Title() in [template['title'] for template in pod_templates])
        # only enabled POD templates are returned
        transaction.begin()
        itemTemplate.enabled = False
        itemTemplate.reindexObject()
        transaction.commit()
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        pod_templates = json["extra_include_pod_templates"]
        self.assertEqual(len(pod_templates), 6)
        self.assertFalse(itemTemplate.Title() in [template['title'] for template in pod_templates])

    def test_restapi_config_extra_include_searches(self):
        """@config extra_include=searches"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        endpoint_url_pattern = "{0}/@config?config_id={1}&extra_include={2}"
        endpoint_url = endpoint_url_pattern.format(self.portal_url, cfg_id, "searches")
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        searches = json["extra_include_searches"]
        self.assertEqual(searches[0]["id"], u"searchmyitems")
        self.assertEqual(searches[0]["@type"], u"DashboardCollection")
        # include_items is set to False specifically
        self.assertFalse("items" in searches[0])
        searchmyitems = cfg.searches.searches_items.searchmyitems
        self.assertTrue(searchmyitems.Title() in [search['title'] for search in searches])

        # only enabled DashboardCollections are returned
        transaction.begin()
        searchmyitems.enabled = False
        searchmyitems.reindexObject()
        transaction.commit()
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        searches = json["extra_include_searches"]
        self.assertNotEqual(searches[0]["id"], u"searchmyitems")
        self.assertEqual(searches[0]["@type"], u"DashboardCollection")
        self.assertFalse(searchmyitems.Title() in [search['title'] for search in searches])

    def test_restapi_config_extra_include_proposing_groups(self):
        """@config extra_include=proposing_groups"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        endpoint_url_pattern = "{0}/@config?config_id={1}&extra_include={2}"
        endpoint_url = endpoint_url_pattern.format(self.portal_url, cfg_id, "proposing_groups")
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        orgs = json["extra_include_proposing_groups"]
        self.assertEqual(orgs[0]['UID'], self.developers_uid)
        self.assertEqual(orgs[1]['UID'], self.vendors_uid)

    def test_restapi_config_extra_include_associated_groups(self):
        """@config extra_include=associated_groups"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        endpoint_url_pattern = "{0}/@config?config_id={1}&extra_include={2}"
        endpoint_url = endpoint_url_pattern.format(self.portal_url, cfg_id, "associated_groups")
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        orgs = json["extra_include_associated_groups"]
        self.assertEqual(orgs[0]['UID'], self.developers_uid)
        self.assertEqual(orgs[1]['UID'], self.vendors_uid)

    def test_restapi_config_extra_include_groups_in_charge(self):
        """@config extra_include=groups_in_charge"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        self._enableField('groupsInCharge')
        transaction.commit()
        endpoint_url_pattern = "{0}/@config?config_id={1}&extra_include={2}"
        endpoint_url = endpoint_url_pattern.format(self.portal_url, cfg_id, "groups_in_charge")
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        orgs = json["extra_include_groups_in_charge"]
        self.assertEqual(orgs[0]['UID'], self.developers_uid)
        self.assertEqual(orgs[1]['UID'], self.vendors_uid)

    def test_restapi_config_extra_include_fullobjects(self):
        """@config extra_include_fullobjects"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        self._enableField(('category', 'groupsInCharge'))
        cfg_id = cfg.getId()
        transaction.commit()
        endpoint_url_pattern = "{0}/@config?config_id={1}&extra_include={2}" \
            "&extra_include={3}&extra_include={4}&extra_include={5}" \
            "&extra_include={6}&extra_include={7}"
        endpoint_url = endpoint_url_pattern.format(
            self.portal_url,
            cfg_id,
            "categories",
            "pod_templates",
            "searches",
            "associated_groups",
            "groups_in_charge",
            "proposing_groups")
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        # by default, we only get summarized versions
        self.assertFalse("@components" in json['extra_include_categories'][0])
        self.assertFalse("@components" in json['extra_include_pod_templates'][0])
        self.assertFalse("@components" in json['extra_include_searches'][0])
        self.assertFalse("@components" in json['extra_include_associated_groups'][0])
        self.assertFalse("@components" in json['extra_include_groups_in_charge'][0])
        self.assertFalse("@components" in json['extra_include_proposing_groups'][0])
        # parameter "extra_include_xxx_fullobjects" will get full serialized versions
        endpoint_url += "&extra_include_categories_fullobjects"
        endpoint_url += "&extra_include_pod_templates_fullobjects"
        endpoint_url += "&extra_include_searches_fullobjects"
        endpoint_url += "&extra_include_associated_groups_fullobjects"
        endpoint_url += "&extra_include_groups_in_charge_fullobjects"
        endpoint_url += "&extra_include_proposing_groups_fullobjects"
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertTrue("@components" in json['extra_include_categories'][0])
        self.assertTrue("@components" in json['extra_include_pod_templates'][0])
        self.assertTrue("@components" in json['extra_include_searches'][0])
        self.assertTrue("@components" in json['extra_include_associated_groups'][0])
        self.assertTrue("@components" in json['extra_include_groups_in_charge'][0])
        self.assertTrue("@components" in json['extra_include_proposing_groups'][0])


def test_suite():
    from unittest import TestSuite, makeSuite

    suite = TestSuite()
    # add a prefix to avoid every PM tests to be executed
    suite.addTest(makeSuite(testServiceConfig, prefix="test_restapi_"))
    return suite



# -*- coding: utf-8 -*-

from imio.helpers.cache import get_plone_groups_for_user
from plone import api
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.user import BaseSerializer as BaseUserSerializer
from plonemeeting.restapi.interfaces import IPMRestapiLayer
from plonemeeting.restapi.serializer.base import BaseSerializeToJson
from Products.CMFCore.interfaces._tools import IMemberData
from Products.CMFPlone.utils import safe_unicode
from zope.component import adapter
from zope.interface import implementer


class PMBaseUserSerializer(BaseUserSerializer, BaseSerializeToJson):

    def __call__(self):
        result = super(PMBaseUserSerializer, self).__call__()
        self.tool = api.portal.get_tool("portal_plonemeeting")

        # call _init so elements like self.fullobjects are initialized
        self._init()

        # call _after__call__ that manages additional_values and extra_includes
        result = self._after__call__(self.context, result)
        return result

    def _configs_for_user(self):
        """ """
        cfgs = getattr(self, "cfgs", [])
        if not cfgs:
            # remove AUTHENTICATED_USER during adopt_user
            auth_user = self.request.get("AUTHENTICATED_USER")
            if auth_user:
                self.request["AUTHENTICATED_USER"] = None
            with api.env.adopt_user(username=self.context.id):
                self.cfgs = self.tool.getActiveConfigs()
                cfgs = self.cfgs
            if auth_user:
                self.request["AUTHENTICATED_USER"] = auth_user
        return cfgs

    def _available_extra_includes(self, result):
        """ """
        result["@extra_includes"] = [
            "groups",
            "app_groups",
            "configs",
            "categories",
            "classifiers"]
        return result

    def _extra_include(self, result):
        extra_include = self._get_asked_extra_include()
        if "groups" in extra_include:
            suffixes = self.get_param("suffixes", default=[], extra_include_name="groups")
            orgs = self.tool.get_orgs_for_user(
                user_id=self.context.id, suffixes=suffixes, the_objects=True)
            result["extra_include_groups"] = []
            for org in orgs:
                serializer = self._get_serializer(org, "groups")
                result["extra_include_groups"].append(serializer())
            result["extra_include_groups_items_total"] = len(orgs)

        if "app_groups" in extra_include:
            result["extra_include_app_groups"] = [
                {"token": group.id,
                 "title": safe_unicode(group.getProperty("title"))}
                for group in self.tool.get_plone_groups_for_user(
                    user_id=self.context.id, the_objects=True)]

        if "configs" in extra_include:
            result["extra_include_configs"] = []
            cfgs = self._configs_for_user()
            for cfg in cfgs:
                serializer = self._get_serializer(cfg, "configs")
                result["extra_include_configs"].append(serializer())
            result["extra_include_configs_items_total"] = len(cfgs)

        if "categories" in extra_include:
            result["extra_include_categories"] = {}
            cfgs = self._configs_for_user()
            config_ids = self.get_param("configs", default=[], extra_include_name="categories")
            for cfg in cfgs:
                cfg_id = cfg.getId()
                if config_ids and cfg_id not in config_ids:
                    continue
                result["extra_include_categories"][cfg_id] = []
                categories = cfg.getCategories(userId=self.context.id)
                for category in categories:
                    serializer = self._get_serializer(category, "categories")
                    result["extra_include_categories"][cfg_id].append(serializer())

        if "classifiers" in extra_include:
            result["extra_include_classifiers"] = {}
            cfgs = self._configs_for_user()
            config_ids = self.get_param("configs", default=[], extra_include_name="classifiers")
            for cfg in cfgs:
                cfg_id = cfg.getId()
                if config_ids and cfg_id not in config_ids:
                    continue
                result["extra_include_classifiers"][cfg_id] = []
                classifiers = cfg.getCategories(catType='classifiers', userId=self.context.id)
                for classifier in classifiers:
                    serializer = self._get_serializer(classifier, "classifiers")
                    result["extra_include_classifiers"][cfg_id].append(serializer())

        return result


@implementer(ISerializeToJson)
@adapter(IMemberData, IPMRestapiLayer)
class SerializeUserToJson(PMBaseUserSerializer):
    pass


@implementer(ISerializeToJsonSummary)
@adapter(IMemberData, IPMRestapiLayer)
class SerializeUserToJsonSummary(PMBaseUserSerializer):
    pass



# -*- coding: utf-8 -*-

from imio.restapi.services.infos import InfosGet
from plone import api


class PMInfosGet(InfosGet):
    """ """

    def _stats_types_queries(self):
        queries = {}
        tool = api.portal.get_tool("portal_plonemeeting")
        # types MeetingConfig
        mConfigs = tool.getActiveConfigs(check_using_groups=False)
        queries["MeetingConfig"] = {
            "portal_type": "MeetingConfig",
            "review_state": "active",
        }
        meeting_types = [cfg.getMeetingTypeName() for cfg in mConfigs]
        # types Meeting
        queries["Meeting"] = {"portal_type": meeting_types}
        item_types = [cfg.getItemTypeName() for cfg in mConfigs]
        # types MeetingItem
        queries["MeetingItem"] = {"portal_type": item_types}
        # types annex
        queries["annex"] = {"portal_type": "annex"}
        # types annexDecision
        queries["annexDecision"] = {"portal_type": "annexDecision"}
        # types advices (meetingadvice, meetingadvicefinances, ...)
        for advice_type in tool.getAdvicePortalTypes(as_ids=True):
            queries[advice_type] = {"portal_type": advice_type}
        return queries

    def _packages_names(self):
        """To be overrided !!!
           Returns list of package names."""
        return ["imio.restapi", "Products.PloneMeeting", "plonemeeting.restapi"]



# -*- coding: utf-8 -*-

from datetime import datetime
from plonemeeting.restapi.services.get import UID_REQUIRED_ERROR
from plonemeeting.restapi.services.get import UID_WRONG_TYPE_ERROR
from plonemeeting.restapi.tests.base import BaseTestCase
from plonemeeting.restapi.utils import IN_NAME_OF_CONFIG_ID_UNAUTHORIZED
from plonemeeting.restapi.utils import IN_NAME_OF_UNAUTHORIZED
from plonemeeting.restapi.utils import UID_NOT_ACCESSIBLE_ERROR
from plonemeeting.restapi.utils import UID_NOT_ACCESSIBLE_IN_NAME_OF_ERROR
from plonemeeting.restapi.utils import UID_NOT_FOUND_ERROR
from Products.PloneMeeting.config import NO_TRIGGER_WF_TRANSITION_UNTIL
from Products.PloneMeeting.tests.PloneMeetingTestCase import DEFAULT_USER_PASSWORD

import transaction


class testServiceGetUid(BaseTestCase):
    """@get endpoint, let's get element based on UID."""

    def setUp(self):
        """ """
        super(testServiceGetUid, self).setUp()
        # especially necessary for branch 4.1.x where proposingGroup/category
        # was mixed and MeetingItem.getCategory would return the proposingGroup or the category
        self._enableField('category')
        self.changeUser("pmManager")
        self.item1 = self.create(
            "MeetingItem",
            proposingGroup=self.developers_uid,
            externalIdentifier="EX123",
        )
        self.item1_uid = self.item1.UID()
        self.item2 = self.create("MeetingItem", proposingGroup=self.vendors_uid)
        self.item2_uid = self.item2.UID()
        self.meeting = self.create("Meeting", date=datetime(2021, 9, 23, 10, 0))
        self.meeting_uid = self.meeting.UID()
        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        transaction.abort()

    def test_restapi_get_uid_required(self):
        """The 'UID' or 'uid' parameter must be given"""
        endpoint_url = "{0}/@get".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 400, response.content)
        self.assertEqual(
            response.json(), {u"message": UID_REQUIRED_ERROR, u"type": u"BadRequest"}
        )
        endpoint_url += "?UID={0}".format(self.item1_uid)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)

    def test_restapi_uid_not_required_if_external_id(self):
        """
        if 'externalIdentifier' or 'external_id' parameter is given, UID become optional
        """
        endpoint_url = "{0}/@get?external_id=EX123".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        self.assertEqual(self.item1_uid, response.json()["UID"])

        endpoint_url = "{0}/@get?externalIdentifier=EX123".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        self.assertEqual(self.item1_uid, response.json()["UID"])

    def test_restapi_get_uid_not_found(self):
        """When given UID does not exist"""
        endpoint_url = "{0}/@get?UID=unknown".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(),
            {u"message": UID_NOT_FOUND_ERROR % "unknown", u"type": u"BadRequest"},
        )

    def test_restapi_get_uid_not_accessible(self):
        """When given UID exists but current user can not access it"""
        endpoint_url = "{0}/@get?UID={1}".format(self.portal_url, self.item2_uid)
        # pmCreator1 can not access item with proposingGroup 'vendors'
        self.api_session.auth = ("pmCreator1", DEFAULT_USER_PASSWORD)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(),
            {
                u"message": UID_NOT_ACCESSIBLE_ERROR % (
                    self.item2_uid, self.meetingConfig.getId(), "pmCreator1"),
                u"type": u"BadRequest",
            },
        )

    def _check_get_uid_endpoint(self, obj, endpoint_name="@get", query=""):
        """ """
        obj_uid = obj.UID()
        endpoint_url = "{0}/{1}?UID={2}{3}".format(
            self.portal_url, endpoint_name, obj_uid, query
        )
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json["id"], obj.getId())
        self.assertEqual(json["UID"], obj_uid)
        # by default, no items
        self.assertFalse("items" in json)
        return json

    def test_restapi_get_uid(self):
        """When given UID is accessible, it is returned"""
        self._check_get_uid_endpoint(obj=self.item1)

    def test_restapi_get_uid_item(self):
        """There is a @item convenience endpoint that is just a shortcut to @get"""
        self._check_get_uid_endpoint(obj=self.item1, endpoint_name="@item")

    def test_restapi_get_uid_item_wrong_type(self):
        """@item endpoint is supposed to return an item, so if we receive an UID
           of another type, we raise"""
        endpoint_url = "{0}/@item?UID={1}".format(
            self.portal_url, self.portal.Members.pmManager.UID()
        )
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(), {u"message": UID_WRONG_TYPE_ERROR, u"type": u"BadRequest"}
        )

    def test_restapi_get_uid_meeting(self):
        """There is a @meeting convenience endpoint that is just a shortcut to @get"""
        json = self._check_get_uid_endpoint(
            obj=self.meeting, endpoint_name="@meeting", query="&include_base_data=true")
        # date is included in base_data
        self.assertEqual(json['date'], u'2021-09-23T10:00:00')

    def test_restapi_get_uid_meeting_wrong_type(self):
        """@meeting endpoint is supposed to return a meeting, so if we receive an UID
           of another type, we raise"""
        endpoint_url = "{0}/@meeting?UID={1}".format(
            self.portal_url, self.portal.Members.pmManager.UID()
        )
        response = self.api_session.get(endpoint_url)
        self.assertEqual(
            response.json(), {u"message": UID_WRONG_TYPE_ERROR, u"type": u"BadRequest"}
        )

    def test_restapi_get_uid_fullobjects(self):
        """By default summary is returned except when "fullobject" is given"""
        endpoint_url = "{0}/@get?UID={1}".format(
            self.portal_url, self.item1_uid
        )
        response = self.api_session.get(endpoint_url)
        json = response.json()
        # summary data
        self.assertEqual(
            sorted(json.keys()),
            [u'@extra_includes', u'@id', u'@type', u'UID', u'created',
             u'description', u'enabled', u'id',
             u'modified', u'review_state', u'title'])
        # passing an extra_include will make it use the fullobjects serializer
        endpoint_url += "&extra_include=proposing_group"
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(
            sorted(json.keys()),
            [u'@extra_includes', u'@id', u'@type', u'UID', u'created',
             u'extra_include_proposing_group', u'id', u'modified',
             u'review_state', u'title'])
        # fullobject is possible too
        endpoint_url += "&fullobjects"
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertTrue("extra_include_proposing_group" in json)
        self.assertTrue("externalIdentifier" in json)
        self.assertTrue("itemReference" in json)

    def test_restapi_get_uid_in_name_of(self):
        """Check when using parameter in_name_of"""
        # config_id is no more required (was the case before)
        endpoint_url = "{0}/@get?UID={1}&in_name_of=pmCreator1".format(
            self.portal_url, self.item1_uid
        )
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        self.assertEqual(response.json()["UID"], self.item1_uid)

        # try to get an inaccessible element
        endpoint_url = endpoint_url.replace(self.item1_uid, self.item2_uid)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 400, response.content)
        self.assertEqual(
            response.json(),
            {
                u"message": UID_NOT_ACCESSIBLE_IN_NAME_OF_ERROR % (
                    self.item2_uid, self.meetingConfig.getId(), "pmCreator1", "pmManager"),
                u"type": u"BadRequest",
            },
        )
        # must be MeetingManager to use in_name_of
        self.api_session.auth = ("pmCreator1", DEFAULT_USER_PASSWORD)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 401, response.content)
        self.assertEqual(
            response.json(),
            {
                u"message": IN_NAME_OF_UNAUTHORIZED % ("pmCreator1", "pmCreator1"),
                u"type": u"Unauthorized",
            },
        )
        # when using in_name_of, the queried configs will be restricted to the
        # MeetingConfigs the power user may access
        self.api_session.auth = ("pmManager2", DEFAULT_USER_PASSWORD)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 400, response.content)
        self.assertEqual(
            response.json(),
            {
                u"message": UID_NOT_ACCESSIBLE_IN_NAME_OF_ERROR % (
                    self.item2_uid, self.meetingConfig.getId(), "pmCreator1", "pmManager2"),
                u"type": u"BadRequest",
            },
        )
        # when using a config_id the power user is not MeetingManager for,
        # we get a clear message
        cfg_id = self.meetingConfig.getId()
        response = self.api_session.get(endpoint_url + "&config_id={0}".format(cfg_id))
        self.assertEqual(response.status_code, 401, response.content)
        self.assertEqual(
            response.json(),
            {
                u"message": IN_NAME_OF_CONFIG_ID_UNAUTHORIZED % (
                    "pmManager2", cfg_id, "pmCreator1"),
                u"type": u"Unauthorized",
            },
        )

    def test_restapi_get_uid_extra_include_pod_templates(self):
        """Test the extra_include=pod_templates."""
        # MeetingItem
        endpoint_url = "{0}/@get?UID={1}&extra_include=pod_templates".format(
            self.portal_url, self.item1_uid
        )
        response = self.api_session.get(endpoint_url)
        json = response.json()
        item_pod_template_json = json["extra_include_pod_templates"][0]
        self.assertEqual(item_pod_template_json[u'@type'], u'ConfigurablePODTemplate')
        self.assertEqual(item_pod_template_json[u'id'], u'itemTemplate')
        self.assertEqual(item_pod_template_json[u'outputs'][0][u'format'], u'odt')
        # Meeting
        endpoint_url = "{0}/@get?UID={1}&extra_include=pod_templates".format(
            self.portal_url, self.meeting_uid
        )
        response = self.api_session.get(endpoint_url)
        json = response.json()
        meeting_pod_template_json = json["extra_include_pod_templates"][0]
        self.assertEqual(meeting_pod_template_json[u'@type'], u'ConfigurablePODTemplate')
        self.assertEqual(meeting_pod_template_json[u'id'], u'agendaTemplate')
        self.assertEqual(meeting_pod_template_json[u'outputs'][0][u'format'], u'odt')

    def test_restapi_get_uid_extra_include_annexes(self):
        """Test the extra_include=annexes."""
        # configure item1 and meeting
        self._enable_annex_config(self.item1, param="publishable")
        self._enable_annex_config(self.meeting, param="publishable")
        # MetingItem
        item_annex = self.addAnnex(self.item1, publishable=True)
        # add a second annex but it will not be retrieved
        self.addAnnex(self.item1)
        # Meeting
        meeting_annex = self.addAnnex(self.meeting, publishable=True)
        # add a second annex but it will not be retrieved
        self.addAnnex(self.meeting)

        # MeetingItem, just get the publishable annexes and include file
        endpoint_url_pattern = "{0}/@get?UID={1}&extra_include=annexes" \
            "&extra_include_annexes_fullobjects" \
            "&extra_include_annexes_publishable=true" \
            "&extra_include_annexes_metadata_fields=file"
        endpoint_url = endpoint_url_pattern.format(
            self.portal_url, self.item1_uid
        )
        response = self.api_session.get(endpoint_url)
        json = response.json()
        item_annexes_json = json["extra_include_annexes"]
        self.assertEqual(len(item_annexes_json), 1)
        self.assertEqual(item_annexes_json[0][u'@type'], u'annex')
        self.assertEqual(item_annexes_json[0][u'UID'], item_annex.UID())
        self.assertEqual(item_annexes_json[0][u'file'][u'filename'], u'FILE.txt')

        # Meeting
        endpoint_url = endpoint_url_pattern.format(
            self.portal_url, self.meeting_uid
        )
        response = self.api_session.get(endpoint_url)
        json = response.json()
        meeting_annexes_json = json["extra_include_annexes"]
        self.assertEqual(len(meeting_annexes_json), 1)
        self.assertEqual(meeting_annexes_json[0][u'@type'], u'annex')
        self.assertEqual(meeting_annexes_json[0][u'UID'], meeting_annex.UID())
        self.assertEqual(meeting_annexes_json[0][u'file'][u'filename'], u'FILE.txt')

        # getting item or meeting annexes without fullobjects also includes
        # the annex file as annex file is returned in any case
        endpoint_url_pattern = "{0}/@get?UID={1}&extra_include=annexes"
        endpoint_url = endpoint_url_pattern.format(
            self.portal_url, self.item1_uid
        )
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        item_annexes_json = json["extra_include_annexes"]
        self.assertEqual(len(item_annexes_json), 2)
        self.assertEqual(item_annexes_json[0][u'@type'], u'annex')
        self.assertEqual(item_annexes_json[0][u'UID'], item_annex.UID())
        self.assertEqual(item_annexes_json[0][u'file'][u'filename'], u'FILE.txt')

    def test_restapi_get_uid_extra_include_linked_items(self):
        """Test the extra_include=linked_items."""
        cfg = self.meetingConfig
        self._removeConfigObjectsFor(cfg)
        cfg.setItemManualSentToOtherMCStates(('itemcreated', ))
        cfg2 = self.meetingConfig2
        cfg2Id = cfg2.getId()
        # make sure items sendable to cfg2
        cfg.setMeetingConfigsToCloneTo(
            ({'meeting_config': cfg2Id,
              'trigger_workflow_transitions_until': NO_TRIGGER_WF_TRANSITION_UNTIL}, ))
        # setup, create item, delay it, send it to cfg2
        # auto linked items
        self.changeUser('pmManager')
        item = self.create('MeetingItem', decision=self.decisionText)
        item_uid = item.UID()
        meeting = self.create('Meeting')
        self.presentItem(item)
        self.decideMeeting(meeting)
        self.do(item, 'delay')
        new_item = item.get_successors()[0]
        new_item_uid = new_item.UID()
        new_item.setOtherMeetingConfigsClonableTo((cfg2Id, ))
        cfg2_item = new_item.cloneToOtherMeetingConfig(cfg2Id)
        cfg2_item_uid = cfg2_item.UID()
        # set manually linked items
        item2 = self.create('MeetingItem', decision=self.decisionText)
        item2_uid = item2.UID()
        item3 = self.create('MeetingItem', decision=self.decisionText)
        item3_uid = item3.UID()
        item.setManuallyLinkedItems((item2.UID(), item3.UID()))
        transaction.commit()

        # by default we get the auto linked items
        endpoint_url_pattern = "{0}/@get?UID={1}&extra_include=linked_items"
        endpoint_url = endpoint_url_pattern.format(self.portal_url, item_uid)
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 2)
        self.assertEqual(
            [linked_item['UID'] for linked_item in json["extra_include_linked_items"]],
            [new_item_uid, cfg2_item_uid])
        # we may filter values by providing some catalog indexes
        # for example get only the cfg2 item, we can use real index name or easy name
        filter_endpoint_url = endpoint_url + "&extra_include_linked_items_type={0}".format(
            cfg2.getItemTypeName())
        response = self.api_session.get(filter_endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 1)
        self.assertEqual(
            [linked_item['UID'] for linked_item in json["extra_include_linked_items"]],
            [cfg2_item_uid])
        # several filters
        filter_endpoint_url = filter_endpoint_url + "&extra_include_linked_items_review_state={0}".format(
            cfg2_item.query_state())
        response = self.api_session.get(filter_endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 1)
        self.assertEqual(
            [linked_item['UID'] for linked_item in json["extra_include_linked_items"]],
            [cfg2_item_uid])

        # we may ask specific linked items: "manual", "predecessor",
        # "predecessors", "successors" and every_successors"
        # manual
        endpoint_url += "&extra_include_linked_items_mode=manual"
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 2)
        self.assertEqual(
            [linked_item['UID'] for linked_item in json["extra_include_linked_items"]],
            [item2_uid, item3_uid])
        # predecessor, no predecessor for item
        endpoint_url = endpoint_url.replace("manual", "predecessor")
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 0)
        self.assertEqual(json["extra_include_linked_items"], [])
        # get predecessor for cfg2_item
        endpoint_url = endpoint_url.replace(item_uid, cfg2_item_uid)
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 1)
        self.assertEqual(
            [linked_item['UID'] for linked_item in json["extra_include_linked_items"]],
            [new_item_uid])
        # predecessors
        endpoint_url = endpoint_url.replace("predecessor", "predecessors")
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 2)
        self.assertEqual(
            [linked_item['UID'] for linked_item in json["extra_include_linked_items"]],
            [item_uid, new_item_uid])
        # successors, no successors for cfg2_item
        endpoint_url = endpoint_url.replace("predecessors", "successors")
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 0)
        self.assertEqual(json["extra_include_linked_items"], [])
        # get successors for item
        endpoint_url = endpoint_url.replace(cfg2_item_uid, item_uid)
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 1)
        self.assertEqual(
            [linked_item['UID'] for linked_item in json["extra_include_linked_items"]],
            [new_item_uid])
        # every_successors
        endpoint_url = endpoint_url.replace("successors", "every_successors")
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual(json['extra_include_linked_items_items_total'], 2)
        self.assertEqual(
            [linked_item['UID'] for linked_item in json["extra_include_linked_items"]],
            [new_item_uid, cfg2_item_uid])
        # the extra_include_linked_items_fullobjects works as well, by default we get the summary
        self.assertFalse('proposingGroup' in json["extra_include_linked_items"][0])
        endpoint_url += "&extra_include_linked_items_fullobjects"
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertTrue('proposingGroup' in json["extra_include_linked_items"][0])

    def test_restapi_get_uid_include_choices_for(self):
        """Test parameter include_choices_for that receives a list of field names
           and will append the related vocabulary values to the response."""
        # MeetingItem + include_choices on organization (DX)
        endpoint_url_pattern = "{0}/@get?UID={1}&include_choices_for=category" \
            "&include_choices_for=associatedGroups" \
            "&include_choices_for=title" \
            "&extra_include=proposing_group" \
            "&extra_include_proposing_group_include_choices_for=item_advice_states" \
            "&extra_include_proposing_group_include_choices_for=organization_type"

        endpoint_url = endpoint_url_pattern.format(
            self.portal_url, self.item1_uid)
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertEqual([elt['token'] for elt in json["category__choices"]],
                         [u'development', u'events', u'research'])
        self.assertEqual([elt['title'] for elt in json["associatedGroups__choices"]],
                         [u'Developers', u'Vendors'])
        # fields that do not have a vocabulary are ignored
        self.assertFalse("title__choices" in json)
        # field organization.item_advice_states is not readable so not returned
        self.assertFalse("item_advice_states__choices" in
                         json["extra_include_proposing_group"])
        # but field organization_type is readable so included
        self.assertEqual(
            json["extra_include_proposing_group"]["organization_type__choices"],
            [{u'token': u'default', u'title': u'D\xe9faut'}])

        # query the MeetingConfig (include_choices on an AT)
        endpoint_url_pattern = "{0}/@get?UID={1}&include_choices_for=usedItemAttributes"
        endpoint_url = endpoint_url_pattern.format(
            self.portal_url, self.meetingConfig.UID())
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertTrue(
            {u'title': u'Description (description)', u'token': u'description'}
            in json["usedItemAttributes__choices"])
        self.assertTrue(
            {u'title': u'Classifier (classifier)', u'token': u'classifier'}
            in json["usedItemAttributes__choices"])
        self.assertTrue(
            {u'title': u'Checklist (textCheckList)', u'token': u'textCheckList'}
            in json["usedItemAttributes__choices"])

    def test_restapi_extra_include_config(self):
        """Test parameter `extra_include` with value `config`"""
        endpoint_url = "{0}/@get?UID={1}&extra_include=config".format(
            self.portal_url, self.item1_uid
        )
        response = self.api_session.get(endpoint_url)
        json = response.json()
        self.assertTrue("extra_include_config" in json.keys())
        self.assertEqual("MeetingConfig", json["extra_include_config"]["@type"])
        self.assertEqual("plonemeeting-assembly", json["extra_include_config"]["id"])
        self.assertEqual("PloneMeeting assembly", json["extra_include_config"]["title"])


def test_suite():
    from unittest import TestSuite, makeSuite

    suite = TestSuite()
    # add a prefix to avoid every PM tests to be executed
    suite.addTest(makeSuite(testServiceGetUid, prefix="test_restapi_"))
    return suite



# -*- coding: utf-8 -*-

from plone import api
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plonemeeting.restapi.tests.base import BaseTestCase

import requests
import transaction


class testServicePMInfos(BaseTestCase):
    """ """

    def tearDown(self):
        self.api_session.close()

    def test_restapi_infos_endpoint(self):
        """@infos"""
        endpoint_url = "{0}/@infos".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        # return versions and connected_user
        self.assertTrue(u"Products.PloneMeeting" in json["packages"])
        self.assertTrue(u"plonemeeting.restapi" in json["packages"])
        self.assertTrue(u"imio.restapi" in json["packages"])
        self.assertEqual(json["connected_user"], u"pmManager")

        # when not connected
        self.api_session.auth = None
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        # return versions and connected_user
        self.assertTrue(u"Products.PloneMeeting" in json["packages"])
        self.assertTrue(u"plonemeeting.restapi" in json["packages"])
        self.assertTrue(u"imio.restapi" in json["packages"])
        self.assertEqual(json["connected_user"], None)

    def test_restapi_infos_stats(self):
        """@infos stats"""
        endpoint_url = "{0}/@infos?include_stats=1".format(self.portal_url)
        # only available to Manager
        api.user.grant_roles(TEST_USER_NAME, roles=["Manager"])
        logout()
        login(self.portal, TEST_USER_NAME)
        transaction.commit()
        # stats contains various data
        response = requests.get(
            endpoint_url,
            headers={"Accept": "application/json"},
            auth=(TEST_USER_NAME, TEST_USER_PASSWORD),
        )
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertTrue(u"Meeting" in json["stats"][u"types"])
        self.assertTrue(u"MeetingItem" in json["stats"][u"types"])
        self.assertTrue(u"annex" in json["stats"][u"types"])
        self.assertTrue(u"annexDecision" in json["stats"][u"types"])
        self.assertEqual(json["stats"][u"types"]["MeetingConfig"], 2)


def test_suite():
    from unittest import TestSuite, makeSuite

    suite = TestSuite()
    # add a prefix to avoid every PM tests to be executed
    suite.addTest(makeSuite(testServicePMInfos, prefix="test_restapi_"))
    return suite



# -*- coding: utf-8 -*-

from imio.restapi.serializer.base import DefaultJSONSummarySerializer
from OFS.interfaces import IItem
from plone.restapi.interfaces import ISerializeToJsonSummary
from plonemeeting.restapi.serializer.base import ContentSerializeToJson
from Products.ZCatalog.interfaces import ICatalogBrain
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(ISerializeToJsonSummary)
@adapter(ICatalogBrain, Interface)
class PMBrainJSONSummarySerializer(DefaultJSONSummarySerializer, ContentSerializeToJson):
    """ISerializeToJsonSummary adapter for brain."""

    @property
    def _additional_fields(self):
        """By default add 'UID' to returned data."""
        return ["id", "UID", "enabled", "created", "modified"]

    def _get_metadata_fields_name(self):
        """May be overrided when necessary."""
        name = "metadata_fields"
        extra_include_name = getattr(self, "_extra_include_name", None)
        if extra_include_name is not None:
            name = "extra_include_{0}_metadata_fields".format(extra_include_name)
        return name

    def _init(self):
        """ """
        self.metadata_fields = []
        self.asked_extra_include = []
        self.asked_additional_values = []
        self.asked_includes = []
        self.fullobjects = False

    def __call__(self):
        """ """
        result = super(PMBrainJSONSummarySerializer, self).__call__()

        # call _init so elements like self.fullobjects are initialized
        self._init()

        result = self._after__call__(self.context, result)
        return result


@implementer(ISerializeToJsonSummary)
@adapter(IItem, Interface)
class PMJSONSummarySerializer(PMBrainJSONSummarySerializer):
    """ISerializeToJsonSummary adapter for object (not brain)."""



# -*- coding: utf-8 -*-

from imio.restapi.interfaces import IImioRestapiLayer
from plone.restapi.interfaces import IPloneRestapiLayer


class IPMRestapiLayer(IImioRestapiLayer, IPloneRestapiLayer):
    """Marker interface that defines a browser layer."""



# -*- coding: utf-8 -*-


def post_install(context):
    """Post install script"""
    # Do something at the end of the installation of this package.


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-

from imio.helpers.content import get_vocab
from imio.helpers.content import uuidsToObjects
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plonemeeting.restapi.serializer.base import BaseATSerializeFolderToJson
from plonemeeting.restapi.serializer.summary import PMBrainJSONSummarySerializer
from Products.PloneMeeting.interfaces import IMeetingConfig
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


class BaseSerializeConfigToJson(object):
    """ """

    def _available_extra_includes(self, result):
        """ """
        result["@extra_includes"] = [
            "categories",
            "classifiers",
            "pod_templates",
            "searches",
            "proposing_groups",
            "associated_groups",
            "groups_in_charge"]
        return result

    def _extra_include(self, result):
        """ """
        extra_include = self._get_asked_extra_include()
        if "categories" in extra_include:
            categories = self.context.getCategories(onlySelectable=False)
            result["extra_include_categories"] = []
            for category in categories:
                serializer = self._get_serializer(category, "categories")
                result["extra_include_categories"].append(serializer())
            result["extra_include_categories_items_total"] = len(categories)

        if "classifiers" in extra_include:
            classifiers = self.context.getCategories(catType='classifiers', onlySelectable=False)
            result["extra_include_classifiers"] = []
            for classifier in classifiers:
                serializer = self._get_serializer(classifier, "classifiers")
                result["extra_include_classifiers"].append(serializer())
            result["extra_include_classifiers_items_total"] = len(classifiers)

        if "pod_templates" in extra_include:
            pod_templates = [obj for obj in self.context.podtemplates.objectValues()
                             if getattr(obj, 'enabled', False)]
            result["extra_include_pod_templates"] = []
            for pod_template in pod_templates:
                serializer = self._get_serializer(pod_template, "pod_templates")
                result["extra_include_pod_templates"].append(serializer())
            result["extra_include_pod_templates_items_total"] = len(pod_templates)

        if "searches" in extra_include:
            collections = [obj for obj in self.context.searches.searches_items.objectValues()
                           if (obj.portal_type == 'DashboardCollection' and obj.enabled)]
            result["extra_include_searches"] = []
            for collection in collections:
                serializer = self._get_serializer(collection, "searches")
                result["extra_include_searches"].append(serializer())
            result["extra_include_searches_items_total"] = len(collections)

        if "proposing_groups" in extra_include:
            orgs = self.context.getUsingGroups(theObjects=True)
            result["extra_include_proposing_groups"] = []
            for org in orgs:
                serializer = self._get_serializer(org, "proposing_groups")
                result["extra_include_proposing_groups"].append(serializer())
            result["extra_include_proposing_groups_items_total"] = len(orgs)

        if "associated_groups" in extra_include:
            vocab = get_vocab(
                self.context,
                'Products.PloneMeeting.vocabularies.associatedgroupsvocabulary')
            org_uids = [term.value for term in vocab._terms]
            orgs = uuidsToObjects(org_uids, ordered=True)
            result["extra_include_associated_groups"] = []
            for org in orgs:
                serializer = self._get_serializer(org, "associated_groups")
                result["extra_include_associated_groups"].append(serializer())
            result["extra_include_associated_groups_items_total"] = len(orgs)

        if "groups_in_charge" in extra_include:
            vocab = get_vocab(
                self.context,
                'Products.PloneMeeting.vocabularies.groupsinchargevocabulary')
            org_uids = [term.value for term in vocab._terms]
            orgs = uuidsToObjects(org_uids, ordered=True)
            result["extra_include_groups_in_charge"] = []
            for org in orgs:
                serializer = self._get_serializer(org, "groups_in_charge")
                result["extra_include_groups_in_charge"].append(serializer())
            result["extra_include_groups_in_charge_items_total"] = len(orgs)

        return result


@implementer(ISerializeToJson)
@adapter(IMeetingConfig, Interface)
class SerializeToJson(BaseSerializeConfigToJson, BaseATSerializeFolderToJson):
    """ """


@implementer(ISerializeToJsonSummary)
@adapter(IMeetingConfig, Interface)
class SerializeToJsonSummary(BaseSerializeConfigToJson, PMBrainJSONSummarySerializer):
    """ """



# -*- coding: utf-8 -*-

from plonemeeting.restapi.tests.base import BaseTestCase

import transaction


class testServicePMUsersGet(BaseTestCase):
    """ """

    def setUp(self):
        super(testServicePMUsersGet, self).setUp()
        # restrict self.meetingConfig access to developers
        self.meetingConfig.setUsingGroups([self.developers_uid])
        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    def test_restapi_users_endpoint(self):
        """@users"""
        endpoint_url = "{0}/@users/pmManager".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["username"], u'pmManager')
        self.assertEqual(json["email"], u'pmmanager@plonemeeting.org')

    def test_restapi_users_groups(self):
        """@users?extra_include=groups"""
        endpoint_url = "{0}/@users/pmManager?extra_include=groups".format(
            self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["username"], u'pmManager')
        # extra_include=groups will return organizations of the user (any suffixes)
        self.assertEqual(len(json["extra_include_groups"]), 2)
        org_uids = [org["UID"] for org in json["extra_include_groups"]]
        self.assertTrue(self.developers_uid in org_uids)
        self.assertTrue(self.vendors_uid in org_uids)
        # get only orgs using "creators" suffix
        endpoint_url += "&extra_include_groups_suffixes=creators"
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(len(json["extra_include_groups"]), 1)
        self.assertEqual(json["extra_include_groups"][0]["UID"], self.developers_uid)

    def test_restapi_users_configs(self):
        """@users?extra_include=app_groups"""
        cfg1_id = self.meetingConfig.getId()
        cfg2_id = self.meetingConfig2.getId()
        endpoint_url = "{0}/@users/pmManager?extra_include=configs".format(
            self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["username"], u'pmManager')
        # extra_include=configs will return MeetingConfigs the user may access
        self.assertEqual(len(json["extra_include_configs"]), 2)
        cfg_ids = [cfg["id"] for cfg in json["extra_include_configs"]]
        self.assertEqual(sorted(cfg_ids), sorted([cfg1_id, cfg2_id]))
        # check for pmCreator2 that does not have access to cfg1
        endpoint_url = "{0}/@users/pmCreator2?extra_include=configs".format(self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(len(json["extra_include_configs"]), 1)
        self.assertEqual(json["extra_include_configs"][0]["id"], cfg2_id)

    def test_restapi_users_app_groups(self):
        """@users?extra_include=app_groups"""
        endpoint_url = "{0}/@users/pmManager?extra_include=app_groups".format(
            self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["username"], u'pmManager')
        # extra_include=app_groups will return Plone groups
        group_tokens = [group["token"] for group in json["extra_include_app_groups"]]
        self.assertTrue(self.developers_creators in group_tokens)
        self.assertTrue(self.developers_creators in group_tokens)

    def test_restapi_users_categories(self):
        """@users?extra_include=categories"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        cfg2 = self.meetingConfig2
        cfg2_id = cfg2.getId()
        endpoint_url = "{0}/@users/pmManager?extra_include=categories".format(
            self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["username"], u'pmManager')
        # extra_include=categories will return categories of every active configs
        self.assertEqual(len(json["extra_include_categories"]), 2)
        cfg_cat_ids = [cat["id"] for cat in json["extra_include_categories"][cfg_id]]
        cfg2_cat_ids = [cat["id"] for cat in json["extra_include_categories"][cfg2_id]]
        self.assertTrue("development" in cfg_cat_ids)
        self.assertTrue("deployment" in cfg2_cat_ids)
        # can get categories of only one MeetingConfig
        endpoint_url += "&extra_include_categories_configs={0}".format(cfg_id)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["username"], u'pmManager')
        self.assertEqual(len(json["extra_include_categories"]), 1)

    def test_restapi_users_classifiers(self):
        """@users?extra_include=classifiers"""
        cfg = self.meetingConfig
        cfg_id = cfg.getId()
        cfg2 = self.meetingConfig2
        cfg2_id = cfg2.getId()
        endpoint_url = "{0}/@users/pmManager?extra_include=classifiers".format(
            self.portal_url)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["username"], u'pmManager')
        # extra_include=classifiers will return classifiers of every active configs
        self.assertEqual(len(json["extra_include_classifiers"]), 2)
        cfg_cat_ids = [cat["id"] for cat in json["extra_include_classifiers"][cfg_id]]
        cfg2_cat_ids = [cat["id"] for cat in json["extra_include_classifiers"][cfg2_id]]
        self.assertTrue("classifier1" in cfg_cat_ids)
        self.assertTrue("classifier1" in cfg2_cat_ids)
        # can get classifiers of only one MeetingConfig
        endpoint_url += "&extra_include_classifiers_configs={0}".format(cfg_id)
        response = self.api_session.get(endpoint_url)
        self.assertEqual(response.status_code, 200, response.content)
        json = response.json()
        self.assertEqual(json["username"], u'pmManager')
        self.assertEqual(len(json["extra_include_classifiers"]), 1)


def test_suite():
    from unittest import TestSuite, makeSuite

    suite = TestSuite()
    # add a prefix to avoid every PM tests to be executed
    suite.addTest(makeSuite(testServicePMUsersGet, prefix="test_restapi_"))
    return suite



# -*- coding: utf-8 -*-

from plone.app.textfield.interfaces import IRichTextValue
from plone.dexterity.interfaces import IDexterityContent
from plone.restapi.interfaces import IContextawareJsonCompatible
from plone.restapi.serializer.converters import RichtextDXContextConverter
from plonemeeting.restapi.utils import handle_html
from zope.component import adapter
from zope.interface import implementer


@adapter(IRichTextValue, IDexterityContent)
@implementer(IContextawareJsonCompatible)
class PMRichtextDXContextConverter(RichtextDXContextConverter):

    def __call__(self):
        result = super(PMRichtextDXContextConverter, self).__call__()
        result["data"] = handle_html(self.context, result["data"])
        return result



# -*- coding: utf-8 -*-

from os import path
from plone.restapi.testing import RelativeSession
from plonemeeting.restapi.testing import PM_REST_TEST_PROFILE_FUNCTIONAL
from Products.PloneMeeting import tests as pm_tests
from Products.PloneMeeting.tests.PloneMeetingTestCase import DEFAULT_USER_PASSWORD
from Products.PloneMeeting.tests.PloneMeetingTestCase import PloneMeetingTestCase

import os
import transaction


class BaseTestCase(PloneMeetingTestCase):
    """Base class for defining PM restapi test cases."""

    layer = PM_REST_TEST_PROFILE_FUNCTIONAL

    def setUp(self):
        """ """
        super(BaseTestCase, self).setUp()
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = ("pmManager", DEFAULT_USER_PASSWORD)
        self.changeUser("siteadmin")
        self.getMeetingFolder(userId="pmManager")
        # enable RESTAPI_DEBUG for every tests
        # this will display the result of a restapi call in the log
        os.environ['RESTAPI_DEBUG'] = "True"
        try:
            transaction.commit()
        except Exception:
            transaction.begin()

    def _add_image(self, obj):
        """ """
        file_path = path.join(path.dirname(pm_tests.__file__), "dot.gif")
        data = open(file_path, "r")
        img_id = obj.invokeFactory("Image", id="dot.gif", title="Image", file=data.read())
        img = getattr(obj, img_id)
        return img



# -*- coding: utf-8 -*-

from collective.iconifiedcategory.utils import _categorized_elements
from imio.annex.content.annex import IAnnex
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plonemeeting.restapi.serializer.base import BaseDXSerializeFolderToJson
from plonemeeting.restapi.serializer.summary import PMBrainJSONSummarySerializer
from Products.PloneMeeting.utils import get_dx_field
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface


class BaseSerializeAnnexToJson(object):
    """ """

    def _include_custom(self, obj, result):
        """Include "file" by default."""
        if self.fullobjects or \
           "file" in self.metadata_fields or \
           self.get_param('include_base_data', True):
            field = get_dx_field(obj, "file")
            # serialize the field
            serializer = queryMultiAdapter(
                (field, obj, self.request), IFieldSerializer
            )
            result["file"] = serializer()
        return result

    def _additional_values(self, result, additional_values):
        """Let include every values available from
           parent's categorized_elements."""
        ignored_values = ["allowedRolesAndUsers", "last_updated", "visible_for_groups"]
        parent = self.context.aq_parent
        infos = _categorized_elements(parent)[self.context.UID()]
        values = {
            k: v for k, v in infos.items()
            if k not in result and not (k.endswith("_url") or k in ignored_values) and
            ("*" in additional_values or k in additional_values)
        }

        result.update(values)
        return result


@implementer(ISerializeToJson)
@adapter(IAnnex, Interface)
class SerializeToJson(BaseSerializeAnnexToJson, BaseDXSerializeFolderToJson):
    """ """


@implementer(ISerializeToJsonSummary)
@adapter(IAnnex, Interface)
class SerializeToJsonSummary(BaseSerializeAnnexToJson, PMBrainJSONSummarySerializer):
    """ """
